#include "elem.h"
#include <string.h>
#include <stdio.h>


//#define DEBUG 1    // set this to enable debugging

// menu commands
struct menu_index{
  char first;
  char last; 
  char * prompt;
  char size; 
};

struct MenuRoot{
  struct menu * first;
  struct menu * active;
  struct menu_index major;
  struct menu_index minor;
  char * prompt;
} gMenuRoot =
{
  0,
  0,
  {'1', '9', "%d) \0",1},
  {'a', 'z', "%c) \0",1}
//  {'1', '9', "%c- \0",1},
//  0
};
extern char gDisplay[];
char * elemGetOptionByIndex(struct elem * item, int index)
{
  int tindex;
  char * result = 0;
  char ** option = item->options;
  if(option)
    for(tindex=0;tindex<index;tindex++,option++)
    {
      if(!(result = *option))
        break; 
    }

  return result;
}

int elemGetOptionCount(struct elem * item)
{
  int number=0;
  char ** option=item->options;
  if(option)
  for(number=0;*option;number++)
      option++;

  return number;
}

int elemBufferForDisplay(char * buffer, int buffer_size, struct elem * item, int no_options)
{
  int number_options=0;
  int index=0;
  int pos=0;
  char dynamic_description[buffer_size];
  memset(buffer, 0, sizeof(char)*buffer_size);
  if(item->dynamic_description)
  {
#if DEBUG
    if(item->thing == 0)
      printf("error");
#endif
    item->dynamic_description(dynamic_description, item->thing);
    pos = sprintf(buffer, item->static_description, dynamic_description);
  }
  else
    pos = sprintf(buffer, item->static_description);

  if((no_options==0)&&(number_options = elemGetOptionCount(item)))
  {
    pos += sprintf(buffer+pos, "\n");

    for(index=0; index<number_options;index++)
    {
      pos += sprintf(buffer+pos, 
		     gMenuRoot.minor.prompt,
		     gMenuRoot.minor.first+index);
      pos += sprintf(buffer+pos,
		     "%s ",elemGetOptionByIndex(item,index+1));
    }
  }
  return(pos);
}

int menuGetElemArraySize(struct menu * item)
{
  return(item->elem_array_size);
}

struct elem * menuGetElemByIndex(struct menu * item, int index)
{
  struct elem * result= 0;
  if((index>0) && (index<=menuGetElemArraySize(item)))
    result = &(item->elem_array[index-1]);
  return(result);  
}

int elemEnumed(struct elem * item)
{
  return(item->enumerate);
}


void menuInstall(struct menu * item)
{
  struct menu ** menu_ptr;
  item->next = 0;

  menu_ptr = &(gMenuRoot.first);
  while(*menu_ptr)
    menu_ptr = &((*menu_ptr)->next); 
  *menu_ptr = item;
  gMenuRoot.active = *menu_ptr;

}

struct menu * menuGetFirst(void)
{
  return(gMenuRoot.first);
}

struct menu * menuGetNext(struct menu * item)
{
  return(item->next);
}

char * menuGetKeyName(struct menu * item)
{
  return(item->key_name);
}

void menuSetActive(struct menu * item)
{
  gMenuRoot.active = item;
}

struct menu * menuGetActive(void)
{
  return(gMenuRoot.active);
}

char * menuGetDescription(struct menu * item)
{
  char * value = 0;
  if(item)
    value = item->static_description;
  return value;
}

struct elem * menuGetElemArray(struct menu * item)
{
  return(item->elem_array);
}

int printChar(char thing, int num)
{
  int i;
  for(i=0;i<num;i++)
    putchar(thing);
  return i;
}

int printString(char * thing, int num)
{
  int i;
  int index;
  int total=0;
  int size = strlen(thing);
  for(i=0;i<num;i++)
  {
    for(index=0, total=0;index<size;index++, total++)
      putchar(*(thing+index));
  }
  return(total);
}

int menuPrintElem(int number, char *display)
{
  int dispbs=0;
  int currc=0;
  int currlinec=0;
  char prompt[strlen(gMenuRoot.major.prompt)+gMenuRoot.major.size+1];

  sprintf(prompt, gMenuRoot.major.prompt, number);

  dispbs = strlen(display);
  for(currc=0, currlinec=0;currc<dispbs;currc++)
  {
    if((currlinec >= WIDTH) || (display[currc] == '\n'))
    {
      currlinec = 0;
      if(currc != 0)
      {
    	printChar('\n', 1);
	    if(number>0)
            currlinec += printChar(' ', strlen(prompt));
        if(display[currc] != '\n')
            currlinec += printChar(display[currc],1);

      }
      else
      {
     	 if(number>0)
            currlinec += printString(prompt, 1);
      }
    }
    else
      currlinec += printChar(display[currc], 1);
  }
  printChar('\n', 2);
  return 0;
}

static void menuDrawLine(char thing, int length)
{
  int i=0;
  for(i=0;i<length;i++)
	  putchar(thing);
}

static void menuDrawTextCentered(char border, int length, char * string)
{
  int difference = 0;
  int string_length = strlen(string);
  int i;
  
  if(string_length > length)
    string_length = length - 4;     

  difference = length - string_length;

  for(i=0;i<(difference>>1);i++)
  {
    if(i==0)
      menuDrawLine('*', 1);
    else
      menuDrawLine(' ', 1);
 }  

  for(i=0;i<string_length;i++)
    putchar(string[i]);


 
  difference = length - ((difference>>1)+string_length);
  for(i=1;i<difference;i++)
  {
      menuDrawLine(' ', 1);
  }
      menuDrawLine('*', 1);  
}

void menuDisplayPrompt(void)
{
  printString("Command>",1);
}

void menuDisplayActive(void)
{
  int i=0;
  int number=0;
  struct menu * item = gMenuRoot.active;
  menuDrawLine('*',WIDTH);
  putchar('\n');
  menuDrawTextCentered('*', WIDTH, item->static_description);
  putchar('\n');
  menuDrawLine('*',WIDTH);
  putchar('\n');

  for(i=1,number=1;i<=item->elem_array_size;i++)
  {
    if(menuGetElemByIndex(item,i)->active)
    {
        elemBufferForDisplay(gDisplay, BUFFER_SIZE,menuGetElemByIndex(item, i), 0);
        if(menuGetElemByIndex(item,i)->enumerate)        
        {
            menuPrintElem(number, gDisplay);
            number++;
        }
        else
            menuPrintElem(0, gDisplay);
    }
  }
  menuDisplayPrompt();
}

int menuGetIndexByString(const int searchmajor, const char * string)
{
  struct menu_index * indexp;  
  int index=0;
  int length=0;

  if((length= strlen(string))==1)
  {
    indexp = &(gMenuRoot.minor);
    if(searchmajor)
      indexp = &(gMenuRoot.major);

    if((indexp->first <= string[length-1]) &&
       (indexp->last >= string[length-1]))
       index = (string[length-1] - indexp->first + 1);
  }
  return(index);
}

int menuGetStringByIndex(const int searchmajor, 
		const int index,  
		char *string)
{
  int result=0;
  struct menu_index * indexp; 

  if((index>0) && (index<=9))
  {
    indexp = &(gMenuRoot.minor);
    if(searchmajor)
      indexp = &(gMenuRoot.major);
    *string = indexp->first + (index-1);
  }
  return(result);
}

void parseNodeInit(struct parse_node * node, int * looking_for_array)
{
  memset(node, 0, sizeof(struct parse_node));
  node->looking_for_array = looking_for_array;
}


void SetMenu(char * input, void * thing)
{
  int index=0;
  struct menu * next = gMenuRoot.first;
  
  if((gMenuRoot.minor.first <= *input) &&
     (gMenuRoot.minor.last >=  *input))
  {
    index = *input - gMenuRoot.minor.first;
    while(next && (index>0))
    {
      index--;
      next=menuGetNext(next);
    }

    if(index==0)
    {
      menuSetActive(next);
    }
  }
}


int ListMenus(char * output, void * thing)
{
  int value =0;
  struct menu * item;
  char * name=0;
  int i=0;
  int pos=0;

  // Grab the global menu node, and start listing the options
  for(item=menuGetFirst(),i=0;item;i++, item=menuGetNext(item))
  {
    name = menuGetDescription(item);
    if(name)
    {
      pos += sprintf(output+pos,gMenuRoot.minor.prompt,gMenuRoot.minor.first+i);
      pos += sprintf(output+pos, "%s ", name);
    }
#ifdef DEBUG
    else
      sprintf(output, "%s", "Either label or name corrupt for menu item\n");
#endif
  }
  return value;
}

void parseCommandString(struct parse_node * node, const char * command)
{
  int length = strlen(command);
  int cindex = 0;
  int pindex = 0;
  int looking_for_count=0;
  int done_looking = 0;
  int * looking_for_array = node->looking_for_array;
  memset(node->command, 0, sizeof(node->command)+sizeof(int));
  
  looking_for_count = *looking_for_array;

  for(cindex=0, pindex=0;
      ((cindex<length) && (pindex<MAX_COMM_SIZE));
      cindex++,pindex++)
  {
    node->command[pindex] = command[cindex];
    if(node->command[pindex] == ' ')
    {
      node->command[pindex] = '\0';
    }
    else if(!done_looking)
    {
      looking_for_count--;
      if(looking_for_count == 0)
      {
        node->command[++pindex] = '\0';
        if(*++looking_for_array)
          looking_for_count = *looking_for_array;
	else
	  done_looking = 1;
      }
    }
  }
}

char * parseGetNext(struct parse_node * node)
{
  int index=0;
  char * next=0;
  for(index=node->index;
      index<MAX_COMM_SIZE;
      index++)
  {
    if(node->command[index] != 0)
    {
      next = &(node->command[index]);
#ifdef DEBUG
      printf("parser found : %s\n", next);
#endif
      while(node->command[index++] != 0);
      node->index = index;
      break;
    }
  }
  return next;
}

int menuExecCommand(struct parse_node* node)
{
  char command[MAX_COMM_SIZE+MAX_COMM_SIZE];
  char * next=0;
  char * option=0;
  int index=0;
  struct menu * page = menuGetActive();
  struct elem * elem = 0;
  int result=0;
  int i = 0;
  int j = 1;

  memset(command, 0, MAX_COMM_SIZE+MAX_COMM_SIZE);

  // get major
  if((next = parseGetNext(node)))
  {
    if((index = menuGetIndexByString(1,next)))
    {
      while((elem=menuGetElemByIndex(page,j)) && (i<index))
      {
        j++;
        if(elem->active)     // all this active business should be broken out into a separate function
            if(elemEnumed(elem))
              if(++i>=index)
	           break; 
      }  
    }
  }

  // if we found the major element, then look for the minor element
  // if we find it, then lets see if we can get an option from it 
  if(elem)
  {
    if((next = parseGetNext(node)))
    {    
      if((index = menuGetIndexByString(0,next)))
      {
        if(elemGetOptionCount(elem))
	  option = elemGetOptionByIndex(elem, index);
	else
	  option = next;
      }
    }
  }
  
  // if we got the option, keep cruisin...
  if(option)
  {
    // if our menu option is normal, copy it in
    // if it's a special token, then replace it with
    // the user input
    if(strcmp(option, USER_INPUT_TOKEN) == 0)
    {
       if((next = parseGetNext(node)))
	 strcpy(command, next);
    }
    else
      strcpy(command, option);
  }
  // if we got a command string to pass along, let's do that
  // otherwise, just exit...
  if(*command)
  {
#ifdef DEBUG
  printf("command : %u\n", command);
#endif 
  // shove command down the pipe, but check first
  // to make sure that the user has installed a function
  if(elem->validate_input)
  {
     if(elem->validate_input(command, elem->thing)<0)
       result = -1;
  }
  }

  return(result);
}

