#ifndef SYSTEM_DEF_H_
#define SYSTEM_DEF_H_

#include "alt_types.h"
#include "system.h"

// Ethernet Generator Registers
#define EGEN_NUM_PACKET    0x00000000
#define EGEN_CONF_SETTING  0x00000004
#define EGEN_RAND_SEED0    0x00000008
#define EGEN_RAND_SEED1    0x0000000C
#define EGEN_SRC_ADDR0     0x00000010
#define EGEN_SRC_ADDR1     0x00000014
#define EGEN_DEST_ADDR0    0x00000018
#define EGEN_DEST_ADDR1    0x0000001C
#define EGEN_OPERATION     0x00000020
#define EGEN_PKT_TX_CNT    0x00000024


// Ethernet Generator 0 Registers Definition
#define EGEN0_BASE          ETH_GEN_0_BASE
#define EGEN0_NUM_PACKET    EGEN0_BASE + 0x00000000
#define EGEN0_CONF_SETTING  EGEN0_BASE + 0x00000004
#define EGEN0_RAND_SEED0    EGEN0_BASE + 0x00000008
#define EGEN0_RAND_SEED1    EGEN0_BASE + 0x0000000C
#define EGEN0_SRC_ADDR0     EGEN0_BASE + 0x00000010
#define EGEN0_SRC_ADDR1     EGEN0_BASE + 0x00000014
#define EGEN0_DEST_ADDR0    EGEN0_BASE + 0x00000018
#define EGEN0_DEST_ADDR1    EGEN0_BASE + 0x0000001C
#define EGEN0_OPERATION     EGEN0_BASE + 0x00000020
#define EGEN0_PKT_TX_CNT    EGEN0_BASE + 0x00000024

// Ethernet Generator 1 Registers Definition
#define EGEN1_BASE          ETH_GEN_1_BASE
#define EGEN1_NUM_PACKET    EGEN1_BASE + 0x00000000
#define EGEN1_CONF_SETTING  EGEN1_BASE + 0x00000004
#define EGEN1_RAND_SEED0    EGEN1_BASE + 0x00000008
#define EGEN1_RAND_SEED1    EGEN1_BASE + 0x0000000C
#define EGEN1_SRC_ADDR0     EGEN1_BASE + 0x00000010
#define EGEN1_SRC_ADDR1     EGEN1_BASE + 0x00000014
#define EGEN1_DEST_ADDR0    EGEN1_BASE + 0x00000018
#define EGEN1_DEST_ADDR1    EGEN1_BASE + 0x0000001C
#define EGEN1_OPERATION     EGEN1_BASE + 0x00000020
#define EGEN1_PKT_TX_CNT    EGEN1_BASE + 0x00000024

// TSE Register Offsets (General)
#define TSE_REV                    0x00000000
#define TSE_CMD_CONFIG             0x00000008
#define TSE_MAC_0                  0x0000000C
#define TSE_MAC_1                  0x00000010
#define TSE_FRM_LENGTH             0x00000014
#define TSE_RX_SECTION_EMPTY       0x0000001C
#define TSE_RX_SECTION_FULL        0x00000020
#define TSE_TX_SECTION_EMPTY       0x00000024
#define TSE_TX_SECTION_FULL        0x00000028
#define TSE_RX_ALMOST_EMPTY        0x0000002C
#define TSE_RX_ALMOST_FULL         0x00000030
#define TSE_TX_ALMOST_EMPTY        0x00000034
#define TSE_TX_ALMOST_FULL         0x00000038
#define TSE_TX_IPG_LENGTH          0x0000005C
#define TSE_PCS_CONTROL            0x00000200
#define TSE_PCS_STATUS             0x00000204
#define TSE_PCS_PARTNER_ABILITY    0x00000214
#define TSE_PCS_LINK_TIMER_0       0x00000248
#define TSE_PCS_LINK_TIMER_1	   0x0000024C
#define TSE_PCS_IF_MODE            0x00000250

// TSE 0 Registers Definition
#define TSE0_BASE                   TRIPLE_SPEED_ETHERNET_0_BASE
#define TSE0_REV                    TSE0_BASE + 0x00000000
#define TSE0_CMD_CONFIG             TSE0_BASE + 0x00000008
#define TSE0_MAC_0                  TSE0_BASE + 0x0000000C
#define TSE0_MAC_1                  TSE0_BASE + 0x00000010
#define TSE0_FRM_LENGTH             TSE0_BASE + 0x00000014
#define TSE0_RX_SECTION_EMPTY       TSE0_BASE + 0x0000001C
#define TSE0_RX_SECTION_FULL        TSE0_BASE + 0x00000020
#define TSE0_TX_SECTION_EMPTY       TSE0_BASE + 0x00000024
#define TSE0_TX_SECTION_FULL        TSE0_BASE + 0x00000028
#define TSE0_RX_ALMOST_EMPTY        TSE0_BASE + 0x0000002C
#define TSE0_RX_ALMOST_FULL         TSE0_BASE + 0x00000030
#define TSE0_TX_ALMOST_EMPTY        TSE0_BASE + 0x00000034
#define TSE0_TX_ALMOST_FULL         TSE0_BASE + 0x00000038
#define TSE0_TX_IPG_LENGTH          TSE0_BASE + 0x0000005C
#define TSE0_PCS_CONTROL            TSE0_BASE + 0x00000200
#define TSE0_PCS_STATUS             TSE0_BASE + 0x00000204
#define TSE0_PCS_PARTNER_ABILITY    TSE0_BASE + 0x00000214
#define TSE0_PCS_LINK_TIMER         TSE0_BASE + 0x00000248
#define TSE0_PCS_IF_MODE            TSE0_BASE + 0x00000250

// TSE 0 PCS
#define PCS0_BASE					TRIPLE_SPEED_ETHERNET_0_BASE

// TSE 1 Registers Definition
#define TSE1_BASE                   TRIPLE_SPEED_ETHERNET_1_BASE
#define TSE1_REV                    TSE1_BASE + 0x00000000
#define TSE1_CMD_CONFIG             TSE1_BASE + 0x00000008
#define TSE1_MAC_0                  TSE1_BASE + 0x0000000C
#define TSE1_MAC_1                  TSE1_BASE + 0x00000010
#define TSE1_FRM_LENGTH             TSE1_BASE + 0x00000014
#define TSE1_RX_SECTION_EMPTY       TSE1_BASE + 0x0000001C
#define TSE1_RX_SECTION_FULL        TSE1_BASE + 0x00000020
#define TSE1_TX_SECTION_EMPTY       TSE1_BASE + 0x00000024
#define TSE1_TX_SECTION_FULL        TSE1_BASE + 0x00000028
#define TSE1_RX_ALMOST_EMPTY        TSE1_BASE + 0x0000002C
#define TSE1_RX_ALMOST_FULL         TSE1_BASE + 0x00000030
#define TSE1_TX_ALMOST_EMPTY        TSE1_BASE + 0x00000034
#define TSE1_TX_ALMOST_FULL         TSE1_BASE + 0x00000038
#define TSE1_TX_IPG_LENGTH          TSE1_BASE + 0x0000005C
#define TSE1_PCS_CONTROL            TSE1_BASE + 0x00000200
#define TSE1_PCS_STATUS             TSE1_BASE + 0x00000204
#define TSE1_PCS_PARTNER_ABILITY    TSE1_BASE + 0x00000214
#define TSE1_PCS_LINK_TIMER         TSE1_BASE + 0x00000248
#define TSE1_PCS_IF_MODE            TSE1_BASE + 0x00000250

// TSE 1 PCS 
#define PCS1_BASE					SGMII_PCS_0_BASE
#define PCS_CONTROL					0x00000000
#define PCS_STATUS            		0x00000002
#define PCS_DEVICE_ABILITY			0x00000008
#define PCS_PARTNER_ABILITY   		0x0000000A
#define PCS_LINK_TIMER_0      		0x00000024
#define PCS_LINK_TIMER_1	   		0x00000026
#define PCS_IF_MODE           		0x00000028

// Monitor Registers Offset Definitions (General)
#define EMON_NUM_PACKET    			0x00000000
#define EMON_PKT_RX_OK     			0x00000004
#define EMON_PKT_RX_ERR    			0x00000008
#define EMON_BYTE_RX_CNT0  			0x0000000C
#define EMON_BYTE_RX_CNT1  			0x00000010
#define EMON_CYCLE_RX_CNT0 			0x00000014
#define EMON_CYCLE_RX_CNT1 			0x00000018
#define EMON_RX_CTRL_STAT  			0x0000001C


// Ethernet Monitor 0 Registers Definition
#define EMON0_BASE          		ETH_MON_0_BASE
#define EMON0_NUM_PACKET    		EMON0_BASE + 0x00000000
#define EMON0_PKT_RX_OK     		EMON0_BASE + 0x00000004
#define EMON0_PKT_RX_ERR    		EMON0_BASE + 0x00000008
#define EMON0_BYTE_RX_CNT0  		EMON0_BASE + 0x0000000C
#define EMON0_BYTE_RX_CNT1  		EMON0_BASE + 0x00000010
#define EMON0_CYCLE_RX_CNT0 		EMON0_BASE + 0x00000014
#define EMON0_CYCLE_RX_CNT1 		EMON0_BASE + 0x00000018
#define EMON0_RX_CTRL_STAT  		EMON0_BASE + 0x0000001C

// Ethernet Monitor 1 Registers Definition
#define EMON1_BASE          		ETH_MON_1_BASE
#define EMON1_NUM_PACKET    		EMON1_BASE + 0x00000000
#define EMON1_PKT_RX_OK     		EMON1_BASE + 0x00000004
#define EMON1_PKT_RX_ERR    		EMON1_BASE + 0x00000008
#define EMON1_BYTE_RX_CNT0  		EMON1_BASE + 0x0000000C
#define EMON1_BYTE_RX_CNT1  		EMON1_BASE + 0x00000010
#define EMON1_CYCLE_RX_CNT0 		EMON1_BASE + 0x00000014
#define EMON1_CYCLE_RX_CNT1 		EMON1_BASE + 0x00000018
#define EMON1_RX_CTRL_STAT  		EMON1_BASE + 0x0000001C


// SFP Module PHY Registers Definition
#define SFP_SLAVE_ADDR_WRITE            0xAC
#define SFP_SLAVE_ADDR_READ             0xAD
#define SFP_PHY_CONTROL                 0x00
#define SFP_PHY_STATUS                  0x01
#define SFP_PHY_ID0                     0x02
#define SFP_PHY_ID1                     0x03
#define SFP_PHY_AUTONEG_ADVERTISEMENT   0x04
#define SFP_PHY_LINK_PARTNER_ABILITY    0x05
#define SFP_PHY_1000BASET_CONTROL       0x09
#define SFP_PHY_SPECIFIC_CONTROL		0X10
#define SFP_PHY_PHY_SPECIFIC_STAT       0x11
#define SFP_PHY_INTERRUPT_ENA			0x12
#define SFP_PHY_PHY_SPECIFIC_CONTROL    0x14
#define SFP_PHY_EXTENDED_ADDRESS		0x16
#define SFP_PHY_EXT_PHY_SPECIFIC_STAT   0x1B
#define SFP_PHY_EXT_ADDRESS				0x1D
#define SFP_PHY_REG30					0x1E
#define SFPA_SCL                        PIO_0_BASE
#define SFPA_SDA                        PIO_1_BASE
#define SFPB_SCL                        PIO_2_BASE
#define SFPB_SDA                        PIO_3_BASE

#define EGEN_OPERATION_TRANSMIT_START_BIT         0x1
#define EGEN_OPERATION_TRANSMIT_STOP_BIT          0x2
#define EGEN_OPERATION_TRANSMIT_DONE_STATUS_BIT   0x4

#define EMON_RECEIVE_CONTROL_START_BIT            0x1
#define EMON_RECEIVE_CONTROL_STOP_BIT             0x2
#define EMON_RECEIVE_CONTROL_DONE_STATUS_BIT      0x4

#define PCS_LOOPBACK_BASE  				PCS_LOOPBACK_WRAPPER_0_BASE
#define PCS_LOOPBACK_EN					0x0000
#define PCS_LOOPBACK_MAC_DISABLE		0x0002

// Avalon-st loopback
#define AVALON_ST_LOOPBACK_BASE			AVALON_ST_LOOPBACK_SOPC_WRAPPER_0_BASE
#define AVALON_ST_LOOPBACK_ENA			0x0000

#endif /*SYSTEM_DEF_H_*/
