#include "elem.h"
#include "test_control.h"
#include "bool_check.h"
#include "helpers.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern struct gTestStruct * curr_active_results_ptr;
extern struct gTestStruct * testGetNext(struct gTestStruct * test);
extern struct gTestStruct * testGetNext(struct gTestStruct * test);

/* All of these functions are defined in tsemac_menu.c */
extern int displayMacAddrA(char *output, void *thing);
extern int displayMacAddrB(char *output, void *thing);

/* All of these functions are defined in test_menu.c */
extern int displayLinkSpeed(char *output, void *thing);
extern int displayDuplex(char *output, void *thing);
extern int displayPacketLength(char *output, void *thing);
extern int displaySeedValue(char *output, void *thing);
extern int displayPacketData(char *output, void *thing);



static char * error_codes_array[]=
{
    "no errors",
    "monitor finished before generator; please re-download SOF to reset system",
    "monitor never finished; please re-download SOF to reset system"
};

int displayErrorCode(char * output, void *thing)
{
  int pos=0;
  struct gTestStruct * test = (struct gTestStruct *)thing;

    /*
     * Conditions to deal with, found in tse_test.h
     * SUCCESS=0,
     * MON_BEFORE_GEN,
     * MON_NEVER_DONE    
     */
     
  pos = sprintf(output, error_codes_array[test->error_code]);    
    
  return(pos);
}

void listTestOrder(struct gTestStruct * report, int*first, int*current,int* last)
{
  struct gTestStruct * next_report=0;

  if(report==0)
  {
    *first=0;
    *current=0;
    *last=0;
  }
  else
  {
  *current = report->test_number;
  *last=*first=*current;

  next_report=report; 
   
  while(testGetNext(next_report)!=report) 
  {
    next_report=testGetNext(next_report);
    if(next_report->test_control!=NOT_RUN)
    {
        if(next_report->test_number > *last)
            *last = next_report->test_number;
        else
        {
            if(next_report->test_number < *first)
               *first = next_report->test_number;
        }    
    }
  }   
  }
}

int displayReportControl(char *output, void *thing)
{
  int pos=0;
  int first=0;
  int current=0;
  int last=0;
  struct gTestStruct * report = *((struct gTestStruct **)thing);

  listTestOrder(report, &first, &current, &last);
  if((first == current) && (current==last))
    pos = sprintf(output,"Reports: %d", current);
  else if(first == current)
    pos = sprintf(output,"Reports: ([%d]...%d)", current, last);
  else if(current == last)
    pos = sprintf(output,"Reports: (%d...[%d])", first, current);
  else
    pos = sprintf(output,"Reports: (%d...[%d]...%d)", first, current, last);

    return(pos);
}

int valAndSetReportControl(char *input, void *thing)
{
  int ret_val = TRUE;
  struct gTestStruct * test = *((struct gTestStruct **)thing);
  int num_requested=0;

  if(test==0)
  {
    ret_val = FALSE;
  }
  else
  {
  if(strcmp(input,"prev")==0)
    test = testGetPrevious(test);
  else if(strcmp(input,"next")==0)
    test = testGetNext(test);
  else if((num_requested = atoi(input)))
  {
    int first=0;
    int last=0;
    int current=0;
    struct gtestStruct * (*operation)(struct gTestStruct *);
    listTestOrder(test,&first,&current,&last);
    if((num_requested>=first)&&(num_requested<=last))
    {
      if(num_requested>=current)
      {
        num_requested= num_requested-current;
        operation = (void*)&testGetNext;
      }   
      else
      {
        num_requested=current-num_requested;
        operation = (void*)&testGetPrevious; 
      }
      while(num_requested)
      {
        test=operation(test);
        num_requested--; 
      }
    }
  }else
    return(FALSE);
    
  if(test->test_control != NOT_RUN)
    testResultsSet(test);
  }
  return ret_val;
}

/* Functions to call via Menus */
int displayTestControl(char *output, void *thing)
{ 
  struct gTestStruct * test = (struct gTestStruct *)thing;
  int pos=0;
  int number = test->test_control;
  
  if(number == DONE)
    pos = sprintf(output, "Done");
  else if (number == INTERRUPTED)
    pos = sprintf(output, "Interrupted");
  else if (number == NOT_RUN)
    pos = sprintf(output, "Not Started");
  else if (number == ERROR)
    pos = sprintf(output, "Error");
  else 
    pos = sprintf(output, "Unknown");
  
  if((number == DONE)||(number == INTERRUPTED)||(number == ERROR))
  {
    pos += sprintf(output+pos, " (");
    pos += displayErrorCode(output+pos, thing);
    pos += sprintf(output+pos, ")");
  }
  return(pos);
}


int displayReport(char *output, void *thing)
{ 
  int pos=0;
  char report[256];  /* should be possible to kill this */
  struct gTestStruct ** test = (struct gTestStruct **)thing;
  
  if(*test!=0)
  {
  
  pos += sprintf(output+pos, "-----Test-----\n");
  if(stringFromInt(report, &((*test)->test_number)))
    pos += sprintf(output+pos, "Number: %s\n", report);
  if(displayTestControl(report, *test))
    pos += sprintf(output+pos, "Status: %s\n", report);
  //formatUnsignedUnits(report, test->time,3);
  pos += sprintf(output+pos, "Run Time: %.3f seconds\n", (*test)->time);

  pos += sprintf(output+pos, "-----Link Configuration-----\n");
  displayMacAddrA(report, test);
  pos += sprintf(output+pos, "TSE Sender MAC: %s\n", report);
  displayMacAddrB(report, test);
  pos += sprintf(output+pos, "TSE Receiver MAC: %s\n", report);

  if(displayLinkSpeed(report, test))
    pos += sprintf(output+pos, "Link Speed: %s ", report);

  if(displayDuplex(report, test))
    pos += sprintf(output+pos, "(Duplex: %s)\n", report);

  pos += sprintf(output+pos, "-----Sender-----\n");
  //formatUnsignedUnits(report, test->number_packets,0);
  pos += sprintf(output+pos, "Packets to Send: %u packets\n", (*test)->number_packets); 

  displayPacketLength(report, test);
  pos += sprintf(output+pos, "(length: %s, ", report);

  displayPacketData(report, test);
  pos += sprintf(output+pos, "data type: %s", report);

  if((*test)->packet_data == RANDOM)
  {
    displaySeedValue(report, test);
    pos += sprintf(output+pos, ", seed value: %s)\n", report); 
  }else
    pos += sprintf(output+pos, ")\n"); 
  

  //formatUnsignedUnits(report, test->packets_sent, 0);
  pos += sprintf(output+pos, "Packets Sent: %u packets ", (*test)->packets_sent);

  //formatFloatUnits(report, test->packets_per_second, 3);
  pos += sprintf(output+pos, "(%.3f packets/sec)\n", (*test)->packets_per_second);

  pos += sprintf(output+pos, "-----Receiver-----\n"); 
  //formatUnsignedUnits(report, test->valid_packets, 0);
  pos += sprintf(output+pos, "Packets Received: %u ", (*test)->valid_packets+(*test)->error_packets);
  pos += sprintf(output+pos, "(valid: %u, ", (*test)->valid_packets);  
  pos += sprintf(output+pos, "error: %u)\n", (*test)->error_packets);

  pos += sprintf(output+pos, "Bytes Received: %.0f bytes ", (*test)->data_received);
  pos += sprintf(output+pos, "(rate: %.3f bits/second)\n", (*test)->bps);
  }else{
    pos = sprintf(output, "%s", "No tests run.\n");
  }
  return(pos);
}  

  
/* Menu Creation */

struct elem report_menu[] =
{
  {"report\0","%s",&displayReport,&curr_active_results_ptr,1,0,0,&valAndSetReportControl,TRUE},
  {"reportn",
   "\nReport Control - Prints test results."\
   "\n(Either \"prev\" and \"next\" to scroll through tests"\
   "\n or enter in a test number.)"\
   "\n%s",
   &displayReportControl,
   &curr_active_results_ptr,
   TRUE,
   TRUE,
   (char * []){"prev\0", "next\0",USER_INPUT_TOKEN,0},
   &valAndSetReportControl,
   TRUE,
  },
  LIST_MENUS
};
  
struct menu report = {
   0,
   "report_menu",
   "Report Menu",
   report_menu,
   sizeof(report_menu)/sizeof(struct elem)
};
