#include "bool_check.h"
#include "test_control.h"
#include "hardware_def.h"
#include "tse_control.h"
#include <io.h>
#include <unistd.h>

int tseStopTest(struct gTestStruct * test)
{
    /* to stop, we'll need to hammer some registers... let's do
     * this later on...
     */
/** hocksing **/
  IOWR_32DIRECT(EGEN0_BASE, EGEN_OPERATION, EGEN_OPERATION_TRANSMIT_STOP_BIT);
//  IOWR_32DIRECT(EMON1_BASE, EMON_RX_CTRL_STAT, EMON_RECEIVE_CONTROL_STOP_BIT);
  usleep(100000);
  test->time=getSeconds();
  test->valid_packets = getValidPacketsReceived();
  test->error_packets = getErrorPacketsReceived();
  test->packets_sent = getPacketsSent();
  test->data_received = getBytes() + ((test->valid_packets + test->error_packets) * 4.0);
  test->packets_per_second=(test->valid_packets+test->error_packets)/test->time;
  test->bps=(test->data_received * 8.0)/test->time;

  return(0);
}

void tsePollTest(struct gTestStruct * test)
{
    int status=0;
    int time_out_count = 10;
    int gen_done = FALSE; int mon_done = FALSE;

    /* Check Ethernet Generator to be done */
    status = IORD_32DIRECT(EGEN0_BASE, EGEN_OPERATION);
    if(status & EGEN_OPERATION_TRANSMIT_DONE_STATUS_BIT)
        gen_done = TRUE;

    while(time_out_count--)
    {
        status = IORD_32DIRECT(EMON0_BASE, EMON_RX_CTRL_STAT);
        if(status & EMON_RECEIVE_CONTROL_DONE_STATUS_BIT)
            mon_done = TRUE;

        if((gen_done == FALSE) && (mon_done == TRUE))
        {
            test->test_control = ERROR;
            test->error_code = MON_BEFORE_GEN;
            break;
        }
        else if((gen_done == TRUE) && (mon_done == TRUE))
        {
            test->test_control = DONE;
            test->error_code = SUCCESS;
            break;
        }
        else if((gen_done == TRUE) && (mon_done == FALSE))
        {
            test->test_control = ERROR;
            test->error_code = MON_NEVER_DONE;
            usleep(100000);
        }
        else
            break;
    }
}

int tseStartTest(struct gTestStruct * test)
{
	/*
	 * sticky bit needed because re-initializing the system seems to cause problems.
	 */
	static int do_reset=TRUE;
	int	tech_ena;
	int loopback_mode;
	int ext_loopback_mode;

	/* Configure Link Speed and Duplex */
	if(do_reset==TRUE)
	{
		tseMacReset(TSE0_BASE);
	  	tseMacReset(TSE1_BASE);
	  	do_reset=FALSE;
	}

	tseMacConfigure(TSE0_BASE, PCS0_BASE, test->macaddra_high, test->macaddra_low, test->promis_mode, test->speed, test->link_duplex, test->tech_ena, test->loopback_mode, test->ext_loopback_mode);
	tseMacConfigure(TSE1_BASE, PCS1_BASE, test->macaddrb_high, test->macaddrb_low, test->promis_mode, test->speed, test->link_duplex, test->tech_ena, test->loopback_mode, test->ext_loopback_mode);

	AvalonLoopbackConfig (AVALON_ST_LOOPBACK_BASE, test->loopback_mode, test->ext_loopback_mode, test->port_1_loopback_mode);
	PcsLoopbackConfig (PCS_LOOPBACK_BASE, test->loopback_mode, test->ext_loopback_mode, test->port_1_loopback_mode);

	/* This function could be made more modular... */
	tech_ena = test->tech_ena;
	loopback_mode = test->loopback_mode;
	ext_loopback_mode = test->ext_loopback_mode;

	if ((tech_ena == 0) && ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode != 0)))) {
		sfp_config (test->tech_ena, test->loopback_mode , test->ext_loopback_mode, test->port_1_loopback_mode, test->speed, test->link_duplex);
	}

	if ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode == 2))) {
		restart_auto_negotiation(PCS1_BASE, test->tech_ena);
		restart_auto_negotiation(PCS0_BASE, test->tech_ena);
	}

	// skip status check for 1000x ext-phy loopback, basically this mode is not available
	if (!((tech_ena == 1) && (ext_loopback_mode == 3))){
		StatusCheck(TSE0_BASE, PCS1_BASE, test->tech_ena, test->loopback_mode, test->ext_loopback_mode);
	}

	delay_loop (500000);

	tseGenMonConfigure(test);

	return(0);
}

//jier
void delay_loop (unsigned int delay_val)
{
	int i;
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);
	for(i = 0; i < delay_val; i++);

	return;
}
