#include "bool_check.h"
#include "elem.h"
#include "test_control.h"
#include "unistd.h"
#include "helpers.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


extern struct gTestStruct Gtest;
extern char gDisplay[];
extern struct menu report;
extern char * input_string;

/* Functions to call via Menus */
int stringFromInt(char * output, void * thing)
{
  int pos=0;
  int number = *(int *)thing;
  pos += sprintf(output+pos, "%d", number);
  return pos;
}

int stringFromUint(char * output, void * thing)
{
  int pos=0;
  unsigned int number = *(unsigned int *)thing;
  pos += sprintf(output+pos, "%d", number);
  return pos;
}
int displaySeedValue(char *output, void *thing)
{
  struct gTestStruct * test = *((struct gTestStruct **)thing);
  int pos=0;
  
  //pos = convertHexToString(output, (char*)test->seed_value, 46);  
  pos=sprintf(output, "0x");
  pos+=sprintf(output+pos, "%04x", (test->seed_value_high&0x3fff));
  pos+=sprintf(output+pos, "%08x", test->seed_value_low);
  return(pos);
}

int valAndSetSeedValue(char *input, void *thing)
{
  struct gTestStruct * test = *((struct gTestStruct **)thing);
  int num=0;
  char upper[9];
  int upper_num=0;
  char lower[9];
  int lower_num=0;

  memset(upper, 0, 9);
  memset(lower, 0, 9);

  int ret_val = TRUE;
  
  nullCRLF(input);
  if(validateHexString(input)==FALSE)
    ret_val = FALSE;
  else
  {
    stripLeadingHex(&input);
    num=strlen(input);
    if(num<8)
        lower_num=num;
    else
    {
      lower_num=8;
      num-=lower_num;
      if(num<8)
        upper_num=num;
      else
        upper_num=8;
    }

    if(upper_num)
      strncpy(upper, input, upper_num);
    else 
      strncpy(upper, "00000000", 8); 
    
    strncpy(lower, input+upper_num, lower_num);
        
    test->seed_value_high=strtoul(upper, 0, 16);
    test->seed_value_low=strtoul(lower, 0, 16);
  }    
  return ret_val;
}


int displayNumPackets(char *output, void *thing)
{
  int pos =0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  pos = stringFromInt(output, &(test->number_packets));
  pos += sprintf(output+pos, "  (1 < value < 2^32) ");
  return(pos);
}

int displayPacketLength(char *output, void *thing)
{
  int pos =0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  
  if(test->random_length==TRUE)
     pos = sprintf(output, "random");
  else
  {    
    if (test->packet_length > 0)
        pos = stringFromUint(output, &(test->packet_length));
    else
        pos += sprintf(output+pos, "  (64 < value < 9600) ");
  }
  return(pos);
}

int displayPacketData(char *output, void *thing)
{ 
  int pos=0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  int number = test->packet_data;
  pos = sprintf(output, ((number==INCREMENT) ? "increment" : "random"));
  return(pos);
}

int valAndSetPacketData(char *input, void *thing)
{
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  //int *number = (int *)thing; 
  struct menu * menu_ptr=0;
  char * menu_name=0;

  if(strcmp(input, "increment")==0)
    test->packet_data = INCREMENT;
  else
    test->packet_data = RANDOM;
    
    menu_ptr = menuGetFirst();
    while(menu_ptr)
    {
        menu_name = (char*)menuGetKeyName(menu_ptr);
        if(strcmp(menu_name, "test_menu")==0)
            break;
        menu_ptr = menuGetNext(menu_ptr);
    }
    if(menu_ptr)
    {
        int size=0;
        struct elem * item=0;
        size=menuGetElemArraySize(menu_ptr);   
        while(size--)
        {
            item = menuGetElemByIndex(menu_ptr, size);
            if(strcmp(item->key_name, "seed")==0)
            {
                if(test->packet_data == RANDOM) 
                    item->active = TRUE;
                else
                    item->active = FALSE;
                break;
            }
        }
    }
    else
    {
        printf("Could not find test_menu, so we're dying now...\n");
        exit(1);
  }
  return 1;
}

int valAndSetLinkSpeed(char * input, void * thing)
{ 
  unsigned int number = 0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  
  if((number = atoi(input)))
  {
    if((number==10)||(number==100)||(number==1000))
    {
        test->speed = number; 
        if(number == 1000)
            test->link_duplex = FULL_DUPLEX;
    }
  }

  return number;
}

int displayDuplex(char *output, void *thing)
{
  int pos=0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);
 
  if(test->link_duplex == HALF_DUPLEX)
    pos = sprintf(output, "half");
  else
    pos = sprintf(output, "full");

  return(pos);     
}

void valAndSetDuplex(char * input, void *thing)
{
  int ret_val = 1;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  if(strcmp(input, "half")==0)
  {
    if(test->speed == 1000)
        ret_val = 0;
    else
        test->link_duplex=HALF_DUPLEX;      
  }
  else if(strcmp(input, "full")==0)    
    test->link_duplex=FULL_DUPLEX;
  else
    ret_val = 0;    
}

int displayLinkSpeed(char *output, void *thing)
{
  int pos;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

 
  pos = stringFromUint(output, &(test->speed));
  return(pos);     
}



int valAndSetPacketLength(char * input, void *thing)
{
  unsigned int number = 0;
  int ret_val = 1;
  struct gTestStruct * test = *((struct gTestStruct **)thing);
  int base=10;
  
  if(validateHexString(input))
  {
     stripLeadingHex(&input);
     base=16;
  }


  if((number = strtoul(input, 0, base)))
//  if((number = (unsigned int) atoi(input)))
  {
    if((number>=64)&&(number<=9600))
        test->packet_length = number; 
    else
       ret_val = -1;
    test->random_length=FALSE;
  }
  else if(strncmp(input, "random", strlen("random"))==0)
  {
    test->random_length=TRUE;
  }
  else
    ret_val = 0;

  return ret_val;
}

int valAndSetNumberPackets(char* input, void *thing)
{
  unsigned long number = 0;
  struct gTestStruct * test = *((struct gTestStruct **)thing);
  int retval = -1;
  int base=10;
  
  if(validateHexString(input))
  {
     stripLeadingHex(&input);
     base=16;
  }
  
  if((number = strtoul(input, 0, base)))
  {
    if((number>0)||(number<0xffffffff))
    {
      test->number_packets = number; 
      retval = -1;
    }
  } 
  return retval;
}

//jier added Menu Items
//Technology
int technology_enable (char *output, void *thing)
{
  int pos;
  struct gTestStruct * test = *((struct gTestStruct **)thing);

  if(test->tech_ena == 1)
  	pos = sprintf(output, "1000Base-X"); 
  else
  	pos = sprintf(output, "10/100/1000Base-T (SGMII)"); 
    
  return(pos);     
}

int technology_sel (char * input, void * thing)
{
	struct menu * menu_ptr=0;
  	char * menu_name=0;	
	int ret_val = 1;
  	  	
	struct gTestStruct * test = *((struct gTestStruct **)thing);    

	if(strcmp(input, "1000Base-X")==0)
   	{
   	   	test->tech_ena = 1;
   	   	test->speed = 1000;
   	   	test->link_duplex = FULL_DUPLEX;
   	   	
   	   	if ((test->loopback_mode == 3) && (test->ext_loopback_mode == 3)) {	//if set to external PHY loopback?
   	   		//test->loopback_mode = 0;											//set loopback mode to default : no loopback option
   	   		test->ext_loopback_mode = 0;										//set ext_loopback_mode to default : HSMC loopback card
	   	}
   	}
   	else if(strcmp(input, "10/100/1000Base-T (SGMII)")==0)		//default
	{
   		test->tech_ena = 0;
   	}
 	else {
   	   ret_val = 0;
	}     
   	   	
  	//SGMII Link Speed
	menu_ptr = menuGetFirst();
    while(menu_ptr)
    {
        menu_name = (char*)menuGetKeyName(menu_ptr);
        if(strcmp(menu_name, "test_menu")==0)
            break;
        menu_ptr = menuGetNext(menu_ptr);
    }
    if(menu_ptr)
    {
        int size=0;
        struct elem * item=0;
        size=menuGetElemArraySize(menu_ptr);   
        while(size--)
        {
            item = menuGetElemByIndex(menu_ptr, size);
            if(strcmp(item->key_name, "SGMII speed")==0)
            {
                if (test->tech_ena == 1)
                    item->active = FALSE;
                else
                    item->active = TRUE;

                break;
            }
        }
    }
    else
    {
        printf("Could not find test_menu, so we're dying now...\n");
        exit(1);
  	}
  	
  	//1000Base-X Link Speed
	menu_ptr = menuGetFirst();
    while(menu_ptr)
    {
        menu_name = (char*)menuGetKeyName(menu_ptr);
        if(strcmp(menu_name, "test_menu")==0)
            break;
        menu_ptr = menuGetNext(menu_ptr);
    }
    if(menu_ptr)
    {
        int size=0;
        struct elem * item=0;
        size=menuGetElemArraySize(menu_ptr);   
        while(size--)
        {
            item = menuGetElemByIndex(menu_ptr, size);
            if(strcmp(item->key_name, "1000Base-X speed")==0)
            {
                if (test->tech_ena == 0)
                    item->active = FALSE;
                else
                    item->active = TRUE;

                break;
            }
        }
    }
    else
    {
        printf("Could not find test_menu, so we're dying now...\n");
        exit(1);
  	}
  	
    //Jier, Only Full Duplex is support in loopback reference design
 	////Duplex
	//menu_ptr = menuGetFirst();
    //while(menu_ptr)
    //{
    //    menu_name = (char*)menuGetKeyName(menu_ptr);
    //    if(strcmp(menu_name, "test_menu")==0)
    //        break;
    //    menu_ptr = menuGetNext(menu_ptr);
    //}
    //if(menu_ptr)
    //{
    //    int size=0;
    //    struct elem * item=0;
    //    size=menuGetElemArraySize(menu_ptr);   
    //    while(size--)
    //    {
    //        item = menuGetElemByIndex(menu_ptr, size);
    //        if(strcmp(item->key_name, "duplex")==0)
    //        {
    //            if (test->tech_ena == 1)
    //                item->active = FALSE;
    //            else
    //                item->active = TRUE;
    //           
    //            break;
    //        }
    //    }
    //}
    //else
    //{
    //    printf("Could not find test_menu, so we're dying now...\n");
    //    exit(1);
  	//}
  	
  	//External Loopback Menu Active
 	//Copper
	menu_ptr = menuGetFirst();
	while(menu_ptr)
	{
	    menu_name = (char*)menuGetKeyName(menu_ptr);
	    if(strcmp(menu_name, "tse_mac_menu")==0)
	        break;
	    menu_ptr = menuGetNext(menu_ptr);
	}
	if(menu_ptr)
	{
	    int size=0;
	    struct elem * item=0;
	    size=menuGetElemArraySize(menu_ptr);   
	    while(size--)
	    {
	        item = menuGetElemByIndex(menu_ptr, size);
	        if(strcmp(item->key_name, "copper_ext_loopback")==0)
	        {
	            if ((test->loopback_mode == 3) && (test->tech_ena == 0))
	                item->active = TRUE;
	            else
	                item->active = FALSE;
	           
	            break;
	        }
	    }
	}
	else
	{
	    printf("Could not find test_menu, so we're dying now...\n");
	    exit(1);
	}

    //Fiber
	menu_ptr = menuGetFirst();
	while(menu_ptr)
	{
	    menu_name = (char*)menuGetKeyName(menu_ptr);
	    if(strcmp(menu_name, "tse_mac_menu")==0)
	        break;
	    menu_ptr = menuGetNext(menu_ptr);
	}
	if(menu_ptr)
	{
	    int size=0;
	    struct elem * item=0;
	    size=menuGetElemArraySize(menu_ptr);   
	    while(size--)
	    {
	        item = menuGetElemByIndex(menu_ptr, size);
	        if(strcmp(item->key_name, "fiber_ext_loopback")==0)
	        {
	            if ((test->loopback_mode == 3) && (test->tech_ena == 1))
	                item->active = TRUE;
	            else
	                item->active = FALSE;
	           
	            break;
	        }
	    }
	}
	else
	{
	    printf("Could not find test_menu, so we're dying now...\n");
	    exit(1);
	}
  	
	//Option that fiber not available
	if ((test->tech_ena == 1) && (test->ext_loopback_mode == 3))		//when swtich tech, and loopback mode is ext phy loopback which is not supported for fiber
   	{	
	   	test->loopback_mode = 0;									//set the loopback mode to default condition - no loopback condition
   	}
	
	return ret_val;  	
}

struct gTestStruct * curr_active_test_ptr;
struct gTestStruct * curr_active_results_ptr;

struct gTestStruct * testGetActive(void)
{
  return(curr_active_test_ptr);
}

struct gTestStruct * testSetActive(struct gTestStruct * test)
{
  curr_active_test_ptr=test;
  return(curr_active_test_ptr);
}


struct gTestStruct * testGetNext(struct gTestStruct * test)
{
  return(test->next);   
}


struct gTestStruct * testGetPrevious(struct gTestStruct * test)
{
  struct gTestStruct * before;
  before = test;
  
  while(before->next != test)
    before=before->next;
  
  return(before);   
}

void testResultsSet(struct gTestStruct * test)
{    
    curr_active_results_ptr = test;
}

int testReset(struct gTestStruct * test, int size)
{
  int count=0;
 
  for(count=0;count<size;count++)
  {

    memset(test+count, 0, sizeof(struct gTestStruct)); 

    if(count+1<size)
      (test+count)->next = (test+count+1);
    else
      (test+count)->next = test;
    (test+count)->test_number=1;
    (test+count)->speed=1000;
    (test+count)->link_duplex = FULL_DUPLEX;
    (test+count)->packet_length=1518;
    (test+count)->random_length=FALSE;
    (test+count)->number_packets=1000000;
    (test+count)->packet_data=1;
    (test+count)->test_control=NOT_RUN;
    (test+count)->error_code=SUCCESS;
    (test+count)->number=0;
    (test+count)->seed_value_high=0x2914;
    (test+count)->seed_value_low=0x54e59c61;
    (test+count)->macaddra_high=0x0000EE11;
    (test+count)->macaddra_low=0x22334450;
    //(test+count)->macaddrb_high=0x0000EE22;
    //(test+count)->macaddrb_low=0x33445560;
    (test+count)->macaddrb_high=0x0000EE11;			//temp so that no need to retype everytime
    (test+count)->macaddrb_low=0x22334450;
    (test+count)->promis_mode=0;					//default disable Promis mode
    (test+count)->tech_ena=0;
    (test+count)->loopback_mode=0;
	(test+count)->ext_loopback_mode=0;
	(test+count)->port_1_loopback_mode=0;
  }
  
  testSetActive(test);
  testResultsSet(0);
  
  return 1;
}

int testSetNext(struct gTestStruct * curr, struct gTestStruct * next)
{
  int test_num = curr->test_number+1;
  struct gTestStruct * next_link = next->next;
  memcpy(next, curr, sizeof(struct gTestStruct));
  //next = next_link;

  curr_active_test_ptr = next;

  // setup some values, to not mess with the user
  curr_active_test_ptr->next=next_link;
  curr_active_test_ptr->test_number=test_num; 
  curr_active_test_ptr->test_control=NOT_RUN;
  curr_active_test_ptr->error_code=SUCCESS;
  curr_active_test_ptr->data_received=0;
  curr_active_test_ptr->error_packets=0;
  curr_active_test_ptr->packets_sent=0;
  curr_active_test_ptr->packets_per_second=0;
  
  return TRUE;
}



static int testDoPeriodicUpdate(int reset, struct gTestStruct *test)
{
  static int packets_per_elem;
  static int update_area;
  static int previous_packets;
  unsigned int packets_sent;
  char * ptr = gDisplay;
  char * spacer = "";
  char * border = "|";
  char * pointer = ">";
  char * tail = "=";
  int num=0;
  int pos=0;


  if(reset==TRUE)
  {
    previous_packets=0;  
    return 0;
  }
  
  update_area = (WIDTH - (2* strlen(spacer)) - (2*strlen(border)));
  packets_per_elem = test->number_packets / update_area;    
  packets_sent = (unsigned int) getPacketsSent();
  num = packets_sent / packets_per_elem;

  pos = sprintf(ptr, "%s%s", spacer, border);
  
  
  while(update_area>0)
  {
    num--;
    update_area--;
    if(num>0)
      pos+=sprintf(ptr+pos, "%s", tail);    
    else if(num==0)
      pos+=sprintf(ptr+pos, "%s", pointer);
    else
      pos+=sprintf(ptr+pos, " ");
  }
  pos+=sprintf(ptr+pos, "%s%s\n", border, spacer);

  sprintf(ptr+pos, "Pkts Sent: %u of %u (~ %u packets/sec)\n\n", 
         packets_sent, test->number_packets,
         packets_sent-previous_packets);
  
  previous_packets = packets_sent;
  
  printString(ptr,1);
  
  return 0;
}


int startTest(char *input, void *thing)
{
    struct gTestStruct * test = *((struct gTestStruct **)thing);
    int return_value = DONE;
    char done = '9';
    
    testDoPeriodicUpdate(TRUE, test);
    tseStartTest(test);
    while(test->test_control == ACTIVE)
    {
        if((done = getc(stdin))!=EOF)
        {
            printString("\nTest Interrupted\n", 1);    
            test->test_control = INTERRUPTED;
        }
        usleep(1000000);
        /* Update the display with intermediate results */
       testDoPeriodicUpdate(FALSE, test);
       tsePollTest(test); 
    }
    tseStopTest(test);
    testResultsSet(test);
    testSetNext(test, testGetNext(testGetActive()));
    menuSetActive(&report);

    return(return_value);
}



/* Menu Creation */

  struct elem test_menu[] =
  {
	  {															//jier, Technology Option
      "tech_ena",
      "\nTechnology Options"\
      "\n"VAL_TAG"%s",
      &technology_enable,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char* []){"10/100/1000Base-T (SGMII)", "1000Base-X\0",0},
      &technology_sel,
      TRUE,
      },	
      //Speed menu for SGMII 
      {
      "SGMII speed",
      "\nLink Speed - Ethernet link speed (Mbits/sec)"\
      "\n"VAL_TAG"%s\0",
      &displayLinkSpeed,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char* []){"10\0", "100\0", "1000\0", 0},
      &valAndSetLinkSpeed,
      TRUE,
      },
      //Speed menu for 1000Base-X
      {
      "1000Base-X speed",
      "\nLink Speed - Ethernet link speed (Mbits/sec)"\
      "\n(Only 1000Mbps is supported)"\
      "\n"VAL_TAG"%s\0",
      &displayLinkSpeed,
      &curr_active_test_ptr,
      FALSE,
      TRUE,
      (char* []){"1000\0", 0},
      &valAndSetLinkSpeed,
      TRUE,
      },
      {
      "duplex",
      "\nLink Configuration- Duplex rate for link"\
      "\n(This Loopback Reference Design Support Full Duplex Mode Only)"\
      "\n"VAL_TAG"%s\0",
      &displayDuplex,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      //(char* []){"half\0", "full\0", 0},		//jier, half duplex should not support in loopback design and it will only cause collision
      (char* []){"full\0", 0},
      (void*)&valAndSetDuplex,
      TRUE,
      },
      {
      "plen",
      "\nPacket Length - Size in bytes (64 < x < 9600 or random)"\
      "\n"VAL_TAG"%s\0",
      &displayPacketLength,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char* []){"64\0", "1518\0", "9600\0", "random\0", USER_INPUT_TOKEN, 0},
      &valAndSetPacketLength,
      TRUE,
      },
      {
      "pnum",
      "\nNumber Packets - Packets to send (0 < x < 4294967296)"\
      "\n"VAL_TAG"%s\0",
      &displayNumPackets,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char* []){"1000000\0", "10000000\0", "1000000000\0", USER_INPUT_TOKEN, 0},
      &valAndSetNumberPackets,
      TRUE,
      },
      {
      "pdata",
      "\nPacket Data - Controls type of data sent for test"\
      "\n"VAL_TAG"%s\0",
      &displayPacketData,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char * []){"increment\0", "random\0",0},
      &valAndSetPacketData,
      TRUE,
      },
      {
      "seed",
      "\nRandom Generator Seed Value (46 Bits- hexadecimal)"\
      "\n"VAL_TAG"%s",
      &displaySeedValue,
      &curr_active_test_ptr,
      FALSE,
      TRUE,
      (char* []){"0x291454E59C61\0",USER_INPUT_TOKEN,0},
      &valAndSetSeedValue,
      TRUE
      },
      {
      "start",
      "\nTest Control - Starts the test",
      0,
      &curr_active_test_ptr,
      TRUE,
      TRUE,
      (char * []){"start (stops on <Enter> during test)",0},
      &startTest,
      TRUE,
      },
      LIST_MENUS
  };

  struct menu command = {
     0,
    "test_menu",
    "Test Menu",
    test_menu,
    sizeof(test_menu)/sizeof(struct elem)
  };

