/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include "hardware_def.h"
#include "alt_2_wire.h"
#include "test_control.h"
#include "tse_control.h"
#include "unistd.h"
#include "bool_check.h"
#include "helpers.h"
#include "io.h"

void sfp_reg_write (int address, int data)
{
    int ret;

    alt_2_wireStart();
    ret = alt_2_wireSendByte(SFP_SLAVE_ADDR_WRITE);
    ret = alt_2_wireSendByte(address);
    ret = alt_2_wireSendByte(data >> 8);
    ret = alt_2_wireSendByte(data & 0xFF);
    alt_2_wireStop();
}

int sfp_reg_read (int address)
{
    int ret;
    int data;


    alt_2_wireStart();
    ret = alt_2_wireSendByte(SFP_SLAVE_ADDR_WRITE);
    ret = alt_2_wireSendByte(address);
    alt_2_wireStart();
    ret = alt_2_wireSendByte(SFP_SLAVE_ADDR_READ);
    data = alt_2_wireReadByte(SEND_ACK);
    ret = alt_2_wireReadByte(SEND_NACK);
    data = ((data << 8) | ret);
    alt_2_wireStop();
    return data;
}


void sfp_config (unsigned int tech_ena, unsigned int loopback_mode , unsigned int ext_loopback_mode, unsigned int port_1_loopback_mode, unsigned int speed, unsigned int duplex)
{
    int status;
    //int readdata;

    alt_two_wire * sfp_serial;
    alt_two_wire serial_wire;
    sfp_serial = &serial_wire;
    alt_2_wireSetDelay(100);

    //reg0 - control register
	//reg4 - Auto-Neg Advertisement Register
	//reg9 - 1000Base-T Control Register
	//reg18 - Interrupt Enable Register
	//reg29 - Extended Address
	//reg30 - Multi Page register (refer datasheet)

    int reg4, reg9, reg16, reg29, reg30;
    int tmp;
   
    // ===================================================
    // Configure the SFP Module A to Auto-Neg at selected mode
    // ===================================================
    sfp_serial->scl_pio = SFPA_SCL;
    sfp_serial->sda_pio = SFPA_SDA;
    alt_2_wireInit( sfp_serial );

    //Reg27
    if (tech_ena == 0) {
	    //tmp = sfp_reg_read (SFP_PHY_EXT_PHY_SPECIFIC_STAT); 
	    //tmp = (tmp & 0xFFF0) | 0x0004;									//enable SGMII without clock	//Reg 27
	    //tmp = (tmp & 0x7DFF) | 0x0000;									//enable Fiber/Copper selection and automatic Medium Register selection
	    //sfp_reg_write (SFP_PHY_EXT_PHY_SPECIFIC_STAT, tmp);				
	    
	    tmp = sfp_reg_read (SFP_PHY_EXT_PHY_SPECIFIC_STAT); 
	    tmp = (tmp & 0xFFF0) | 0x0004;									//enable SGMII without clock	//Reg 27
	    tmp = (tmp & 0x7DFF) | 0x9200;									//disable Fiber/Copper selection and automatic Medium Register selection
	    sfp_reg_write (SFP_PHY_EXT_PHY_SPECIFIC_STAT, tmp);				

	    sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0000);				//enable Copper register space (Base-T)
	    
	    ExtPhyReset();
	}

	// Reg4 and Reg9 and Reg16
	reg4 = sfp_reg_read (SFP_PHY_AUTONEG_ADVERTISEMENT);
	reg9 = sfp_reg_read (SFP_PHY_1000BASET_CONTROL);
	reg16 =	sfp_reg_read(SFP_PHY_SPECIFIC_CONTROL);						//enable packet size up to 10Kbytes FIFO setting for jumbo frame (require for transceiver design)

    if ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode == 2))) {
	    if ((speed == 100)&&(duplex==FULL_DUPLEX))
   		{
	   		reg4 = (reg4 & 0xFE1F) | 0x01E0;				// Enable 10/100 Full/Half Duplex
	   		reg9 = reg9 & 0xE4FF;							// No 1000
   		}
   		else if ((speed == 100)&&(duplex==HALF_DUPLEX))
   		{
	   		reg4 = (reg4 & 0xFE1F) | 0x00A0;				// Enable 10/100 Half Duplex
	   		reg9 = reg9 & 0xE4FF;							// no 1000
   		}
   		else if ((speed == 10)&&(duplex==FULL_DUPLEX))
   		{
	   		reg4 = (reg4 & 0xFE1F) | 0x0060;				// Enable 10 Full/Half Duplex
	   		reg9 = reg9 & 0xE4FF;							// no 1000
   		}
   		else if ((speed == 10)&&(duplex==HALF_DUPLEX))
   		{
	   		reg4 = (reg4 & 0xFE1F) | 0x0020;				// Enable 10 Half Duplex
	   		reg9 = reg9 & 0xE4FF;							// no 1000
   		}
   		else //((speed == 1000)
   		{
	   		reg4 = (reg4 & 0xFE1F) | 0x01E0;				// Enable 10/100 Full/Half Duplex 
	   		reg9 = (reg9 & 0xE4FF) | 0x0300;				// Enable 1000
   		}
	}
	else if ((loopback_mode == 3) && (ext_loopback_mode == 1)) {
		if (speed == 1000) {
			//reg9 = (reg9 & 0xE4FF) | 0x1B00;							//force master mode for 1000
			reg9 = 0x1B00;
		}
		else {
			reg9 = reg9 & 0xE4FF;
		}
	}

    reg16 = reg16 | 0xf000;												//enable packet size up to 10Kbytes FIFO setting for jumbo frame (require for transceiver design)
    sfp_reg_write (SFP_PHY_SPECIFIC_CONTROL,reg16);
	sfp_reg_write (SFP_PHY_AUTONEG_ADVERTISEMENT, reg4);
	sfp_reg_write (SFP_PHY_1000BASET_CONTROL, reg9);

	ExtPhyReset();

	status = 0;

	//Extended PHY Specific Control Register --temp only THIS TO SELECT FIBER/COPPER Register Space
	if ((tech_ena == 0) && ((loopback_mode == 3) && ((ext_loopback_mode == 1) || (ext_loopback_mode == 3)))) {
		sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0001);			//enable Fiber register space (1000Base-X)

		sfp_reg_write (SFP_PHY_CONTROL, 0x0140);					//000, Full, AN Disable

		//External PHY Soft Reset
		ExtPhyReset();

		sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0000);			//enable copper (1000Base-T)
	}
	else if (tech_ena == 0) {
		sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0001);			//enable Fiber register space (1000Base-X)

		sfp_reg_write (SFP_PHY_CONTROL, 0x1140);					//000, Full, AN Enable

		//External PHY Soft Reset
		ExtPhyReset();

		sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0000);		//enable copper (1000Base-T)
	}

	//Reg0
	if ((loopback_mode == 3) && ((ext_loopback_mode == 1) || (ext_loopback_mode == 3))) {
		if ((speed == 1000) && (duplex == FULL_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x0140);			//1000, Full, AN Disable
		}
		else if ((speed == 1000) && (duplex == HALF_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x0040);			//1000, Half, AN Disable
		}
		else if ((speed == 100) && (duplex == FULL_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x2100);			//100, Full, AN Disable
		}
		else if ((speed == 100) && (duplex == HALF_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x2000);			//100, Half, AN Disable
		}
		else if ((speed == 10) && (duplex == FULL_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x0100);			//10, Full, AN Disable
		}
		else if ((speed == 10) && (duplex == HALF_DUPLEX)) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x0000);			//10, Half, AN Disable
		}
	}
	else {
		if (duplex == FULL_DUPLEX) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x1140);
		}
		else {
			sfp_reg_write (SFP_PHY_CONTROL, 0x1240);
		}
	}

	//External PHY Soft Reset
	ExtPhyReset();


	//Reg29 & Reg30
	if ((speed == 1000) && (loopback_mode == 3) && (ext_loopback_mode == 1)) {			//special register write for loopback stub mode
		reg29 = 0x0007;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 7 Force Gigabit Mode
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 | 0x0008;									//Set reg30[3]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Force Gigabit mode

		reg29 = 0x0010;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 16 Test Enable Control
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 | 0x0002;									//Set reg30[1]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Enable Gigabit Stub Loopback

		reg29 = 0x0012;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 18 Miscellaneous Control
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 | 0x0000;									//Set reg30[0]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Disable NEXT Canceller

		//ExtPhyReset();
	}
	else {
		reg29 = 0x0007;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 7 Force Gigabit Mode
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 & 0xFFF7;									//Clear reg30[3]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Not Force Gigabit mode

		reg29 = 0x0010;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 16 Test Enable Control
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 & 0xFFFD;									//Clear reg30[1]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Normal Operation

		reg29 = 0x0012;
		sfp_reg_write (SFP_PHY_EXT_ADDRESS, reg29);				//Page 18 Miscellaneous Control
		reg30 = sfp_reg_read (SFP_PHY_REG30);
		reg30 = reg30 & 0xFFFE;									//Clear reg30[0]
		sfp_reg_write (SFP_PHY_REG30, reg30);					//Enable NEXT Canceller
	}

	status = 0;		//as a temp storage only

	if ((tech_ena == 0) && ((loopback_mode == 3) && (ext_loopback_mode == 3))) {		//External PHY Loopback Enable
		status = sfp_reg_read (SFP_PHY_CONTROL);
		status = status | 0x4000;
		sfp_reg_write (SFP_PHY_CONTROL, status);
	}

    // ===================================================
    // Configure the SFP Module B to Auto-Neg at selected mode
    // ===================================================
    if ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode == 2))) {
    	sfp_serial->scl_pio = SFPB_SCL;
    	sfp_serial->sda_pio = SFPB_SDA;
    	alt_2_wireInit( sfp_serial );

    	//Reg27
    	if (tech_ena == 0) {
    		//tmp = sfp_reg_read (SFP_PHY_EXT_PHY_SPECIFIC_STAT); 
	    	//tmp = (tmp & 0xFFF0) | 0x0004;									//enable SGMII without clock	//Reg 27
	    	//tmp = (tmp & 0x7DFF) | 0x0000;									//enable Fiber/Copper selection and automatic Medium Register selection
	    	//sfp_reg_write (SFP_PHY_EXT_PHY_SPECIFIC_STAT, tmp);				
	    
	    	tmp = sfp_reg_read (SFP_PHY_EXT_PHY_SPECIFIC_STAT); 
	    	tmp = (tmp & 0xFFF0) | 0x0004;									//enable SGMII without clock	//Reg 27
	    	tmp = (tmp & 0x7DFF) | 0x9200;									//disable Fiber/Copper selection and automatic Medium Register selection
	    	sfp_reg_write (SFP_PHY_EXT_PHY_SPECIFIC_STAT, tmp);				

	    	sfp_reg_write (SFP_PHY_EXTENDED_ADDRESS, 0x0000);				//enable Copper register space (Base-T)
	    
	    	ExtPhyReset();
		}

		// Reg4 and Reg9
		reg4 = sfp_reg_read (SFP_PHY_AUTONEG_ADVERTISEMENT);
		reg9 = sfp_reg_read (SFP_PHY_1000BASET_CONTROL);
		reg16 =	sfp_reg_read(SFP_PHY_SPECIFIC_CONTROL);						//enable packet size up to 10Kbytes FIFO setting for jumbo frame (require for transceiver design)

		if ((speed == 100)&&(duplex==FULL_DUPLEX))
   		{
			reg4 = (reg4 & 0xFE1F) | 0x0100;
			reg9 = reg9 & 0xE4FF;
   		}
   		else if ((speed == 100)&&(duplex==HALF_DUPLEX))
   		{
			reg4 = (reg4 & 0xFE1F) | 0x0080;
			reg9 = reg9 & 0xE4FF;
   		}
   		else if ((speed == 10)&&(duplex==FULL_DUPLEX))
   		{
			reg4 = (reg4 & 0xFE1F) | 0x0040;
			reg9 = reg9 & 0xE4FF;
   		}
   		else if ((speed == 10)&&(duplex==HALF_DUPLEX))
   		{
			reg4 = (reg4 & 0xFE1F) | 0x0020;
			reg9 = reg9 & 0xE4FF;
   		}
   		else //((speed == 1000)
   		{
			reg4 = reg4 & 0xFE1F;
			reg9 = (reg9 & 0xE4FF) | 0x0300;
   		}

		reg16 = reg16 | 0xf000;												//enable packet size up to 10Kbytes FIFO setting for jumbo frame (require for transceiver design)
    	sfp_reg_write (SFP_PHY_SPECIFIC_CONTROL,reg16);
		sfp_reg_write (SFP_PHY_AUTONEG_ADVERTISEMENT, reg4);
		sfp_reg_write (SFP_PHY_1000BASET_CONTROL, reg9);

		if (tech_ena==0){
		ExtPhyReset();
		};

		//Reg0
		if (duplex == FULL_DUPLEX) {
			sfp_reg_write (SFP_PHY_CONTROL, 0x1140);
		}
		else {
			sfp_reg_write (SFP_PHY_CONTROL, 0x1240);
		}

		//External PHY Soft Reset
		ExtPhyReset();

    };
    
	return;
};

void restart_auto_negotiation(unsigned int base_address, unsigned int tech_ena) {

	int pcs_ctrl;
	int phy_ctrl;

	alt_two_wire * sfp_serial;
    alt_two_wire serial_wire;
    sfp_serial = &serial_wire;
    alt_2_wireSetDelay(100);

    if (base_address == PCS0_BASE) {
	    //Port 0
	    if (tech_ena == 0)	//Copper Only
	    {
	    	sfp_serial->scl_pio = SFPA_SCL;
    		sfp_serial->sda_pio = SFPA_SDA;
    		alt_2_wireInit( sfp_serial );

    		//restart PHY AN
    		phy_ctrl = sfp_reg_read (SFP_PHY_CONTROL);
    		phy_ctrl = phy_ctrl | 0x0200;
	    	sfp_reg_write (SFP_PHY_CONTROL, phy_ctrl);

	    	ExtPhyReset();
    	}
	    
	    //restart PCS AN
	 	pcs_ctrl = IORD_32DIRECT(base_address,TSE_PCS_CONTROL);
	 	pcs_ctrl = pcs_ctrl |0x0200;									//Enable restart AN
	    IOWR_32DIRECT(base_address, TSE_PCS_CONTROL, pcs_ctrl);
    }
    else {
	    //Port 1
	    if (tech_ena == 0) //Copper Only
	    {
	    	sfp_serial->scl_pio = SFPB_SCL;
    		sfp_serial->sda_pio = SFPB_SDA;
    		alt_2_wireInit( sfp_serial );

    		//restart PHY AN
    		phy_ctrl = sfp_reg_read (SFP_PHY_CONTROL);
    		phy_ctrl = phy_ctrl | 0x0200;
	    	sfp_reg_write (SFP_PHY_CONTROL, phy_ctrl);

	    	ExtPhyReset();
    	}
	    
	    //restart PCS AN
	 	pcs_ctrl = IORD_16DIRECT(base_address,PCS_CONTROL);
	 	pcs_ctrl = pcs_ctrl |0x0200;									//Enable restart AN
	    IOWR_16DIRECT(base_address, PCS_CONTROL, pcs_ctrl);
    }

   	return;
}

void StatusCheck(unsigned int tse_0_base_addr, unsigned int tse_1_base_addr, unsigned int tech_ena, unsigned int loopback_mode, unsigned int ext_loopback_mode)
{

    int pcs_link_chk_0;
    int pcs_an_chk_0;
    int phy_link_chk_0;
    int phy_an_chk_0;

    int pcs_link_chk_1;
    int pcs_an_chk_1;
    int phy_link_chk_1;
    int phy_an_chk_1;

    alt_two_wire * sfp_serial;
    alt_two_wire serial_wire;
    sfp_serial = &serial_wire;
    alt_2_wireSetDelay(100);

    if (loopback_mode == 0)
    {
    	if (tech_ena == 0)	//copper
    	{
    		pcs_link_chk_0 = 1;
			pcs_an_chk_0 = 1;
			phy_link_chk_0 = 1;
			phy_an_chk_0 = 1;
			//phy_link_chk_0 = 0;
			//phy_an_chk_0 = 0;
		}
  		else
  		{
  			pcs_link_chk_0 = 1;
  			pcs_an_chk_0 = 0;
  			phy_link_chk_0 = 0;
  			phy_an_chk_0 = 0;
  		}

	    pcs_link_chk_1 = 0;
    	pcs_an_chk_1 = 0;
    	phy_link_chk_1 = 0;
    	phy_an_chk_1 = 0;
	}
	else if (loopback_mode == 1)
	{
		pcs_link_chk_0 = 0;
	    pcs_an_chk_0 = 0;
	    phy_link_chk_0 = 0;
	    phy_an_chk_0 = 0;

	    pcs_link_chk_1 = 0;
    	pcs_an_chk_1 = 0;
    	phy_link_chk_1 = 0;
    	phy_an_chk_1 = 0;
	}
    else if (loopback_mode == 2)
	{
		pcs_link_chk_0 = 1;
	    pcs_an_chk_0 = 0;
	    phy_link_chk_0 = 0;
	    phy_an_chk_0 = 0;

	    pcs_link_chk_1 = 0;
    	pcs_an_chk_1 = 0;
    	phy_link_chk_1 = 0;
    	phy_an_chk_1 = 0;
	}
	else if (loopback_mode == 3)
	{
		if ((ext_loopback_mode == 0) || (ext_loopback_mode == 3))
		{
			pcs_link_chk_0 = 1;
	    	pcs_an_chk_0 = 0;
	    	phy_link_chk_0 = 0;
	    	phy_an_chk_0 = 0;

	    	pcs_link_chk_1 = 0;
    		pcs_an_chk_1 = 0;
    		phy_link_chk_1 = 0;
    		phy_an_chk_1 = 0;
		}
		else if (ext_loopback_mode == 1)
		{
			if (tech_ena == 0)	//copper
    		{
				pcs_link_chk_0 = 1;
	    		pcs_an_chk_0 = 0;
	    		phy_link_chk_0 = 1;
	    		//phy_link_chk_0 = 0;
	    		phy_an_chk_0 = 0;
    		}
    		else		//Fiber
    		{
	    		pcs_link_chk_0 = 1;
	    		pcs_an_chk_0 = 0;
	    		phy_link_chk_0 = 0;
	    		phy_an_chk_0 = 0;
    		}

	    	pcs_link_chk_1 = 0;
    		pcs_an_chk_1 = 0;
    		phy_link_chk_1 = 0;
    		phy_an_chk_1 = 0;
		}
		else
		{
			pcs_link_chk_0 = 0;
	    	pcs_an_chk_0 = 0;
	    	phy_link_chk_0 = 0;
	    	phy_an_chk_0 = 0;

	    	if (tech_ena == 0)	//copper
    		{
	    		pcs_link_chk_1 = 1;
    			pcs_an_chk_1 = 1;
    			phy_link_chk_1 = 1;
    			phy_an_chk_1 = 1;
    			//phy_link_chk_1 = 0;
    			//phy_an_chk_1 = 0;
			}
			else 	//Fiber
			{
				pcs_link_chk_1 = 1;
    			pcs_an_chk_1 = 0;
    			phy_link_chk_1 = 0;
    			phy_an_chk_1 = 0;
			}
		}
	}

    // Poll TSE0 PCS Status register to see if Link_Up
	if (pcs_link_chk_0 == 1) {
		tseMacLink(tse_0_base_addr);
	}

	if (pcs_an_chk_0 == 1) {
		tseMacAN(tse_0_base_addr);
	}

	if (phy_link_chk_0 == 1) {
		sfp_serial->scl_pio = SFPA_SCL;
    	sfp_serial->sda_pio = SFPA_SDA;
   	 	alt_2_wireInit( sfp_serial );

   	 	ExtPhyLink();
	}

	if (phy_an_chk_0 == 1) {
		sfp_serial->scl_pio = SFPA_SCL;
    	sfp_serial->sda_pio = SFPA_SDA;
   	 	alt_2_wireInit( sfp_serial );

   	 	ExtPhyAN();
	}

    if (pcs_link_chk_1 == 1) {
		tseMacLink(tse_1_base_addr);
	}

	if (pcs_an_chk_1 == 1) {
		tseMacAN(tse_1_base_addr);
	}

	if (phy_link_chk_1 == 1) {
		sfp_serial->scl_pio = SFPB_SCL;
    	sfp_serial->sda_pio = SFPB_SDA;
    	alt_2_wireInit( sfp_serial );

   	 	ExtPhyLink();						//hang here, Jier
	}

	if (phy_an_chk_1 == 1) {
		sfp_serial->scl_pio = SFPB_SCL;
    	sfp_serial->sda_pio = SFPB_SDA;
    	alt_2_wireInit( sfp_serial );

   	 	ExtPhyAN();
	}
}

int tseMacReset(unsigned int base_address)
{
    int status = 0;
    // ===================================================
    // Perform software reset the TSE core
    // ===================================================
    IOWR_32DIRECT(base_address, TSE_CMD_CONFIG, 0x00002000);
    status = 0x2000;
    while (status & 0x2000)
    {
        status = IORD_32DIRECT(base_address,TSE_CMD_CONFIG);
    };
    return 0;
}

//jier
int tseMacLink(unsigned int base_address)
{
    int status = 0;
    // ===================================================
    // Perform Link Status Check on the TSE core
    // ===================================================
    while (status != 0x0004)
    {
	    if (base_address == PCS0_BASE) {
        	status = IORD_32DIRECT(base_address,TSE_PCS_STATUS);
    	}
    	else {
	    	status = IORD_16DIRECT(base_address,PCS_STATUS);
    	}
        status = status & 0x0004;
    };
    return 0;
}

int tseMacAN(unsigned int base_address)
{
    int status = 0;
    // ===============================================================
    // Perform Auto Negotiation Complete Status Check on the TSE core
    // ===============================================================
    while (status != 0x0020)
    {
	    if (base_address == PCS0_BASE) {
        	status = IORD_32DIRECT(base_address,TSE_PCS_STATUS);
    	}
    	else {
	    	status = IORD_16DIRECT(base_address,PCS_STATUS);
    	}
    	status = status & 0x0020;
    };
    return 0;
}

int ExtPhyReset(void)
{
    int status = 0;
    // ===============================================================
    // Perform Reset and Reset Status Check on External PHY (SFP)
    // ===============================================================
    status = sfp_reg_read (SFP_PHY_CONTROL);
    status = status | 0x8000;
    sfp_reg_write (SFP_PHY_CONTROL, status);

    status = 0x8000;
    while (status == 0x8000)
    {
        status = sfp_reg_read (SFP_PHY_CONTROL);
        status = status & 0x8000;
    };
    return 0;
}

int ExtPhyLink(void)
{
    int status = 0;
    // ===============================================================
    // Perform Copper/Fiber Link check on External PHY (SFP) Reg 17
    // ===============================================================
    
    //while (!(status & 0x0004))
    //{
	//    status = sfp_reg_read (SFP_PHY_STATUS); // check copper link
    //};
    
    while (status != 0x0400)
    {
    	status = sfp_reg_read (SFP_PHY_PHY_SPECIFIC_STAT); //check copper link
    	status = status & 0x0400;
	};
    
    return 0;
}

int ExtPhyAN(void)
{
    int status = 0;
    // ===============================================================
    // Perform Copper/Fiber AN Status check on External PHY (SFP) Reg 2
    // ===============================================================
    //while (!(status & 0x0020))
    
    while (status != 0x0020)
    {
	    status = sfp_reg_read (SFP_PHY_STATUS); // check copper AN
	    status = status & 0x0020;
    };
        
    return 0;
}

void tseMacConfigure(unsigned int mac_base_address, unsigned int pcs_base_address, unsigned int mac_high, unsigned int mac_low, unsigned int promis_mode, 
					unsigned int speed, unsigned int duplex, unsigned int tech_ena, unsigned int loopback_mode, unsigned int ext_loopback_mode)
{
	unsigned int if_mode;
	unsigned int link_timer_0;
	unsigned int link_timer_1;
	unsigned int pcs_ctrl;
	unsigned int cmd_config;

    // ===================================================
    // Configure TSE0 & TSE1 MAC registers
    // ===================================================
	IOWR_32DIRECT(mac_base_address, TSE_MAC_0, mac_low);
    IOWR_32DIRECT(mac_base_address, TSE_MAC_1, mac_high);
    IOWR_32DIRECT(mac_base_address, TSE_FRM_LENGTH, 0x00002710);			//This ref design max is 9600, the PCS can support up to 10Kbytes
    IOWR_32DIRECT(mac_base_address, TSE_TX_IPG_LENGTH, 0x0000000C);			//typical is 12 (0xC)
    IOWR_32DIRECT(mac_base_address, TSE_RX_SECTION_EMPTY, 0x00000000);
    IOWR_32DIRECT(mac_base_address, TSE_RX_SECTION_FULL, 0x00000010);
    IOWR_32DIRECT(mac_base_address, TSE_TX_SECTION_EMPTY, 0x00000010);
    IOWR_32DIRECT(mac_base_address, TSE_TX_SECTION_FULL, 0x00000010);
    IOWR_32DIRECT(mac_base_address, TSE_RX_ALMOST_EMPTY, 0x00000008);
    IOWR_32DIRECT(mac_base_address, TSE_RX_ALMOST_FULL, 0x00000008);
    IOWR_32DIRECT(mac_base_address, TSE_TX_ALMOST_EMPTY, 0x00000008);
    IOWR_32DIRECT(mac_base_address, TSE_TX_ALMOST_FULL, 0x00000003);

    //IF Mode Register
    if (tech_ena == 1) {
	    //1000Base-X
	    if_mode = 0x00000000;
	}
	else {
		//1000Base-T (SGMII)
		if ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode == 2))) {
			if_mode = 0x00000003;
		}
		else {
			if (speed == 10) {
				if_mode = 0x00000001;
			}
			else if (speed == 100) {
				if_mode = 0x00000005;
			}
			else {
				if_mode = 0x00000009;
			}
		}
	}

	if (pcs_base_address == PCS0_BASE) {
    	IOWR_32DIRECT(pcs_base_address, TSE_PCS_IF_MODE, if_mode);
	}
	else {
		IOWR_16DIRECT(pcs_base_address, PCS_IF_MODE, if_mode);     //for PCS only
	}

    //Link Timer register
    if (tech_ena == 1) {
    	//10ms link timer for 1000Base-X
    	link_timer_0 = 0x000012D0;
    	link_timer_1 = 0x00000013;
	}
	else {
		//1.6ms link timer for 1000Base-T (SGMII)
		link_timer_0 = 0x00000D40;
		link_timer_1 = 0x00000003;
	}

	if (pcs_base_address == PCS0_BASE) {
		IOWR_32DIRECT(pcs_base_address, TSE_PCS_LINK_TIMER_0, link_timer_0);
    	IOWR_32DIRECT(pcs_base_address, TSE_PCS_LINK_TIMER_1, link_timer_1);
	}
	else {
		IOWR_16DIRECT(pcs_base_address, PCS_LINK_TIMER_0, link_timer_0);
		IOWR_16DIRECT(pcs_base_address, PCS_LINK_TIMER_1, link_timer_1);
	}

    //PCS Control Register
    if (loopback_mode == 1) {								//MAC local loopback OR External PHY loopback
	    pcs_ctrl = 0x00000140;								//Disable AN
   	}
   	else if (loopback_mode == 2) {							//PCS Serial loopback
	   	pcs_ctrl = 0x00004140;								//Disable AN, PCS Loopback Enable
   	}
   	else if (loopback_mode == 0) {							//No loopback
	   	pcs_ctrl = 0x00001340;								//Enable & restart AN, PCS loopback Disable
   	}
   	else {													//External loopback mode	(loopback_mode == 3)
   		if (ext_loopback_mode == 2) {						//Two Ports loopback
	   		pcs_ctrl = 0x00001340; 							//Enable & restart AN, PCS loopback Disable
   		}
   		else {												//HSMC Loopback Card OR Loopback Stub
	   		pcs_ctrl = 0x00000140;							//Disable AN
   		}
	}

	//jier
    //if(tech_ena == 1)
    //{
    //	pcs_ctrl = 0x00000140;		//smartbit did not support AN		//***** Smartbit did not support AN for 1000Base-X??
    //}

	if (pcs_base_address == PCS0_BASE) {
    	IOWR_32DIRECT(pcs_base_address, TSE_PCS_CONTROL, pcs_ctrl);
	}
	else {
		IOWR_16DIRECT(pcs_base_address, PCS_CONTROL, pcs_ctrl);
	}

    //Command Config Register
    cmd_config = 0x00000223;

    if (duplex==HALF_DUPLEX) {
	    cmd_config = cmd_config | 0x00000400;
    }

    if (loopback_mode == 1) {
	    cmd_config = cmd_config | 0x00008000;
	}

	//enable Promiscuous mode to allow same packet without address swap loopback from port 0 to port 1
	if (promis_mode == 1) {
		cmd_config = cmd_config | 0x0010;
	}	
	else {
		cmd_config = cmd_config & 0xFFEF;		
	}

    IOWR_32DIRECT(mac_base_address, TSE_CMD_CONFIG, cmd_config);

    return;
}

void PcsLoopbackConfig (unsigned int base_address, unsigned int loopback_mode, unsigned int ext_loopback_mode, unsigned int port_1_loopback_mode)
{
	int pcs_rx_to_tx_loopback_ena = 0;

	if ((loopback_mode == 3) && (ext_loopback_mode == 2) && (port_1_loopback_mode == 1)) {
		pcs_rx_to_tx_loopback_ena = 1;
	}

	IOWR_16DIRECT(base_address, PCS_LOOPBACK_EN, pcs_rx_to_tx_loopback_ena);

	return;
}

void AvalonLoopbackConfig (unsigned int base_address, unsigned int loopback_mode, unsigned int ext_loopback_mode, unsigned int port_1_loopback_mode)
{
	int mac_rx_to_tx_loopback_ena = 0;

	if ((loopback_mode == 0) || ((loopback_mode == 3) && (ext_loopback_mode == 2) && (port_1_loopback_mode == 0))) {
		mac_rx_to_tx_loopback_ena = 3;					//enable avalon loopback and src & dst address swap
		//mac_rx_to_tx_loopback_ena = 1;				//enable avalon loopback and src & disable address swap
	}

	IOWR_16DIRECT(base_address, AVALON_ST_LOOPBACK_ENA, mac_rx_to_tx_loopback_ena);

	return;
}

int tseGenMonConfigure(struct gTestStruct * test)
{

 unsigned int data=0;
 /* Setup Monitor -
   * Most of the status bits are reset to 0 when you toggle
   * the start bit.
   */
 // comment: Hock Siang
  IOWR_32DIRECT(EMON0_BASE, EMON_NUM_PACKET, test->number_packets);

  DPRINTF("EMON0- Receiving Packets = %u\n", test->number_packets);
  /* Setup Generator */
  IOWR_32DIRECT(EGEN0_BASE, EGEN_NUM_PACKET, test->number_packets);
  DPRINTF("EGEN0- Sending Packets = %u\n", test->number_packets);


  /* If random packet lengths are selected
   * then don't even set a packet length... it's bad form
   */
  if(test->random_length == TRUE)
    data = 0x0001;
  else
    data = (test->packet_length << 1) & 0x7FFE;

  /* Random data pattern */
  if(test->packet_data == RANDOM)
    data |= 0x8000;

  IOWR_32DIRECT(EGEN0_BASE, EGEN_CONF_SETTING, data);
  DPRINTF("EGEN0- Configuration Setting = 0x%x\n", data);

  /* Write the seed registers later... */
  IOWR_32DIRECT(EGEN0_BASE, EGEN_RAND_SEED1, test->seed_value_high);
  DPRINTF("EGEN0- Seed Value High = 0x%x\n", test->seed_value_high);
  IOWR_32DIRECT(EGEN0_BASE, EGEN_RAND_SEED0, test->seed_value_low);
  DPRINTF("EGEN0- Seed Value Low = 0x%x\n", test->seed_value_low);

  /* Need to deal withthe src and dest registers later... */
  IOWR_32DIRECT(EGEN0_BASE, EGEN_SRC_ADDR1, test->macaddra_high);
  DPRINTF("EGEN0- MAC SRC ADDR High = 0x%x\n", test->macaddra_high);
  IOWR_32DIRECT(EGEN0_BASE, EGEN_SRC_ADDR0, test->macaddra_low);
  DPRINTF("EGEN0- MAC SRC ADDR Low = 0x%x\n", test->macaddra_low);
  IOWR_32DIRECT(EGEN0_BASE, EGEN_DEST_ADDR1, test->macaddrb_high);
  DPRINTF("EGEN0- MAC SRC ADDR High = 0x%x\n", 0x0000FFFF);
  IOWR_32DIRECT(EGEN0_BASE, EGEN_DEST_ADDR0, test->macaddrb_low);
  DPRINTF("EGEN0- MAC DST ADDR Low = 0x%x\n", test->macaddrb_low);

  /* Start Operation */
  IOWR_32DIRECT(EMON0_BASE, EMON_RX_CTRL_STAT, 0x00000001);
  DPRINTF("EMON0- RX CTL STAT = 0x%x\n", 0x1);
  usleep(100000);
  IOWR_32DIRECT(EGEN0_BASE, EGEN_OPERATION, 0x00000001);
  DPRINTF("EGEN0- OPERATION = 0x%x\n", 0x1);

  test->test_control = ACTIVE;
  test->error_code = SUCCESS;

  return TRUE;
}

// commented - Change Eth-mon1 tp 0
double getBytes(void)
{
    double bytes=0;
    bytes = ((double)IORD_32DIRECT(EMON0_BASE, EMON_BYTE_RX_CNT1)) * 4294967296.0;
    bytes += (double)IORD_32DIRECT(EMON0_BASE, EMON_BYTE_RX_CNT0);
    return bytes;
}

// commented - Change Eth-mon1 tp 0
float getPacketsReceived(void)
{
    float packets=0;
    packets = (float)IORD_32DIRECT(EMON0_BASE, EMON_PKT_RX_OK);
    return packets;
}

unsigned int getPacketsSent(void)
{
    unsigned int packets=0;
    packets = IORD_32DIRECT(EGEN0_BASE, EGEN_PKT_TX_CNT);
    return packets;
}

// commented Hock Siang -- Change EMON1 to EMON0
unsigned int getValidPacketsReceived(void)
{
    return(IORD_32DIRECT(EMON0_BASE, EMON_PKT_RX_OK));
}

unsigned int getErrorPacketsReceived(void)
{
    return(IORD_32DIRECT(EMON0_BASE, EMON_PKT_RX_ERR));
}

double getSeconds(void)
{
    double cycles;
    double seconds;

    cycles = (double)IORD_32DIRECT(EMON0_BASE, EMON_CYCLE_RX_CNT1) * 4294967296.0;
    cycles += (double)IORD_32DIRECT(EMON0_BASE, EMON_CYCLE_RX_CNT0);
    seconds = cycles / ALT_CPU_FREQ;

    return(seconds);
}



