module avalon_st_loopback_reg (
	CLK,
	RESET,
	ADDRESS,
	WRITE,
	READ,
	WRITEDATA,
	READDATA,
	WAITREQUEST,
	
	AVALON_ST_LOOPBACK_ENA,
	SRC_DST_ADDR_SWAP,
	TX_FWD_CRC
	);
	
input			CLK;
input			RESET;
input	[3:0]	ADDRESS;
input			WRITE;
input			READ;
input	[15:0]	WRITEDATA;
output	[15:0]	READDATA;
output			WAITREQUEST;
	
output			AVALON_ST_LOOPBACK_ENA;
output			SRC_DST_ADDR_SWAP;
output			TX_FWD_CRC;
	
reg				AVALON_ST_LOOPBACK_ENA;
reg				SRC_DST_ADDR_SWAP;
reg				TX_FWD_CRC;
reg		[15:0]	READDATA;
			
assign	WAITREQUEST = 1'b0;

// Register
// ------------------------------------------
//Avalon ST loopback enable
always @ (posedge CLK or posedge RESET)
	if (RESET)
		AVALON_ST_LOOPBACK_ENA <= 1'b1;
	else if (WRITE && ADDRESS == 4'h0)
		AVALON_ST_LOOPBACK_ENA <= WRITEDATA[0];

//SRC and DST Address Swap		
always @ (posedge CLK or posedge RESET)
	if (RESET)
		SRC_DST_ADDR_SWAP <= 1'b0; 		
	else if (WRITE && ADDRESS == 4'h0)
		SRC_DST_ADDR_SWAP <= WRITEDATA[1];

//Forward CRC Enable		
always @ (posedge CLK or posedge RESET)
	if (RESET)
		TX_FWD_CRC <= 1'b0; 						
	else if (WRITE && ADDRESS == 4'h0)
		TX_FWD_CRC <= WRITEDATA[2];
		
// Output MUX of registers into readdata bus
// ------------------------------------------

always @ (READ or ADDRESS or AVALON_ST_LOOPBACK_ENA or SRC_DST_ADDR_SWAP or TX_FWD_CRC)
	begin	
		case ({READ,ADDRESS})
			5'h10: 
				READDATA = {{13{1'b0}}, TX_FWD_CRC, SRC_DST_ADDR_SWAP, AVALON_ST_LOOPBACK_ENA};
			default: 
				READDATA = 16'h0000;
		endcase
	end		
		
endmodule