module avalon_st_loopback_wrapper (

	//Avalon MM
	CLK,
	RESET,
	ADDRESS,
	WRITE,
	READ,
	WRITEDATA,
	READDATA,
	WAITREQUEST,

	//Avalon ST TX
	TX_CLK_IN,
	TX_CLK_OUT,
	TX_SOP_OUT,
	TX_DATA_OUT,
	TX_EOP_OUT,
	TX_ERR_OUT,
	TX_WREN_OUT,
	TX_MOD_OUT,
	TX_CRC_FWD_OUT,
	TX_RDY_IN,

	//Avalon ST RX
	RX_CLK_IN,
	RX_CLK_OUT,
	RX_SOP_IN,
	RX_DATA_IN,
	RX_EOP_IN,
	RX_ERR_IN,
	RX_DVAL_IN,
	RX_MOD_IN,
	RX_RDY_OUT
);


//for 8Bit FIFO MAC
//parameter	data_width = 8;
//parameter	err_width = 6;

//for 32bit FIFO MAC
parameter	data_width = 32;
parameter	err_width = 6;

//for FIFOLESS MAC
//parameter	data_width = 8;
//parameter	err_width = 5;

//Avalon MM
input			CLK;
input			RESET;
input	[3:0]	ADDRESS;
input			WRITE;
input			READ;
input	[15:0]	WRITEDATA;
output	[15:0]	READDATA;
output			WAITREQUEST;

//Avalon ST TX
input						TX_CLK_IN;
output						TX_CLK_OUT;
output						TX_SOP_OUT;
output	[data_width-1:0]	TX_DATA_OUT;
output						TX_EOP_OUT;
output						TX_ERR_OUT;
output						TX_WREN_OUT;
output	[1:0]				TX_MOD_OUT;
output						TX_CRC_FWD_OUT;
input						TX_RDY_IN;

//Avalon ST RX
input						RX_CLK_IN;
output						RX_CLK_OUT;
input						RX_SOP_IN;
input	[data_width-1:0]	RX_DATA_IN;
input						RX_EOP_IN;
input	[err_width-1:0]		RX_ERR_IN;
input						RX_DVAL_IN;
input	[1:0]				RX_MOD_IN;
output						RX_RDY_OUT;

wire	avalon_st_loopback_ena_sig;
wire	src_dst_addr_swap_sig;
wire	tx_forward_crc_sig;

avalon_st_loopback_reg		avalon_st_loopback_reg_u0	(
	.CLK						(CLK),
	.RESET						(RESET),
	.ADDRESS					(ADDRESS),
	.WRITE						(WRITE),
	.READ						(READ),
	.WRITEDATA					(WRITEDATA),
	.READDATA					(READDATA),
	.WAITREQUEST				(WAITREQUEST),
	
	.AVALON_ST_LOOPBACK_ENA		(avalon_st_loopback_ena_sig),
	.SRC_DST_ADDR_SWAP          (src_dst_addr_swap_sig),
	.TX_FWD_CRC					(tx_forward_crc_sig)
	);
	
avalon_st_rx_to_tx_loopaback	#(data_width, err_width)	avalon_st_rx_to_tx_loopaback_u0	(
	.RESET						(RESET),
	
	.AVALON_ST_LOOPBACK_ENA		(avalon_st_loopback_ena_sig),
	.SRC_DST_ADDR_SWAP			(src_dst_addr_swap_sig),
	.TX_FWD_CRC					(tx_forward_crc_sig),
	
	//Avalon ST TX
	.TX_CLK_IN					(TX_CLK_IN),
	.TX_CLK_OUT					(TX_CLK_OUT),
	.TX_SOP_OUT					(TX_SOP_OUT),
	.TX_DATA_OUT				(TX_DATA_OUT),
	.TX_EOP_OUT					(TX_EOP_OUT),
	.TX_ERR_OUT					(TX_ERR_OUT),
	.TX_WREN_OUT				(TX_WREN_OUT),
	.TX_MOD_OUT					(TX_MOD_OUT),
	.TX_CRC_FWD_OUT				(TX_CRC_FWD_OUT),
	.TX_RDY_IN					(TX_RDY_IN),

	//Avalon ST RX
	.RX_CLK_IN					(RX_CLK_IN),
	.RX_CLK_OUT					(RX_CLK_OUT),
	.RX_SOP_IN					(RX_SOP_IN),
	.RX_DATA_IN					(RX_DATA_IN),
	.RX_EOP_IN					(RX_EOP_IN),
	.RX_ERR_IN					(RX_ERR_IN),
	.RX_DVAL_IN					(RX_DVAL_IN),
	.RX_MOD_IN					(RX_MOD_IN),
	.RX_RDY_OUT					(RX_RDY_OUT)
);	


endmodule