`define STRATIXIVGX
//`define	ARRIAIIGX

module avalon_st_rx_to_tx_loopaback (
	RESET,
	
	AVALON_ST_LOOPBACK_ENA,
	SRC_DST_ADDR_SWAP,
	TX_FWD_CRC,
	
	//Avalon ST TX
	TX_CLK_IN,
	TX_CLK_OUT,
	TX_SOP_OUT,
	TX_DATA_OUT,
	TX_EOP_OUT,
	TX_ERR_OUT,
	TX_WREN_OUT,
	TX_MOD_OUT,
	TX_CRC_FWD_OUT,
	TX_RDY_IN,

	//Avalon ST RX
	RX_CLK_IN,
	RX_CLK_OUT,
	RX_SOP_IN,
	RX_DATA_IN,
	RX_EOP_IN,
	RX_ERR_IN,
	RX_DVAL_IN,
	RX_MOD_IN,
	RX_RDY_OUT
);

//for 8Bit FIFO MAC
parameter	data_width = 8;
parameter	err_width = 6;

//for 32bit FIFO MAC
//parameter	data_width = 32;
//parameter	err_width = 6;

//for FIFOLESS MAC
//parameter	data_width = 8;
//parameter	err_width = 5;


input						RESET;
		
input						AVALON_ST_LOOPBACK_ENA;
input						SRC_DST_ADDR_SWAP;
input						TX_FWD_CRC;
	
//Avalon ST TX
input						TX_CLK_IN;
output						TX_CLK_OUT;
output						TX_SOP_OUT;
output	[data_width-1:0]	TX_DATA_OUT;
output						TX_EOP_OUT;
output						TX_ERR_OUT;
output						TX_WREN_OUT;
output	[1:0]				TX_MOD_OUT;
output						TX_CRC_FWD_OUT;
input						TX_RDY_IN;

//Avalon ST RX
input						RX_CLK_IN;
output						RX_CLK_OUT;
input						RX_SOP_IN;
input	[data_width-1:0]	RX_DATA_IN;
input						RX_EOP_IN;
input	[err_width-1:0]		RX_ERR_IN;
input						RX_DVAL_IN;
input	[1:0]				RX_MOD_IN;
output						RX_RDY_OUT;

assign	TX_CLK_OUT = TX_CLK_IN;
assign	RX_CLK_OUT = RX_CLK_IN;

wire	reset_tx_clk_sync;
wire	reset_tx_clk_sync_n;
wire	reset_rx_clk_sync;
wire	reset_rx_clk_sync_n;

assign	reset_tx_clk_sync = !reset_tx_clk_sync_n;
assign	reset_rx_clk_sync = !reset_rx_clk_sync_n;

wire	[31:0]	avalon_rx_data;
wire	[5:0]	avalon_rx_err;

assign	avalon_rx_data = {{32-data_width{1'b0}}, RX_DATA_IN};
assign	avalon_rx_err = {{6-err_width{1'b0}}, RX_ERR_IN};

wire	[47:0]	fifo_wr_data;
wire	[47:0]	fifo_rd_data;

assign	fifo_wr_data =	{
							{4{1'b0}},
							TX_RDY_IN,
							RX_SOP_IN, 
							avalon_rx_data, 
							RX_EOP_IN,
							avalon_rx_err,
							RX_DVAL_IN,
							RX_MOD_IN
						};
						
//Avalon ST TX
reg							TX_SOP_OUT;
reg		[data_width-1:0]	TX_DATA_OUT;
reg							TX_EOP_OUT;
reg							TX_ERR_OUT;
reg							TX_WREN_OUT;
reg		[1:0]				TX_MOD_OUT;
reg							TX_CRC_FWD_OUT;
reg							RX_RDY_OUT;					 

wire	avalon_st_loopback_ena_sync;
wire	src_dst_addr_swap_sync;							//jier, address swap not yet implement

reg		[47:0]	fifo_wr_data_d1, fifo_wr_data_d2, fifo_wr_data_d3, fifo_wr_data_d4, fifo_wr_data_d5, fifo_wr_data_d6,
				fifo_wr_data_d7, fifo_wr_data_d8, fifo_wr_data_d9, fifo_wr_data_d10, fifo_wr_data_d11, fifo_wr_data_d12;

reg		[3:0]	count;
reg		[3:0]	addr_swap_count;
reg				addr_swap_ena;
reg				addr_swap_ena_d1;

reg		[47:0]	addr_1, addr_2, addr_3, addr_4, addr_5, addr_6, addr_7, addr_8, addr_9, addr_10, addr_11, addr_12;
reg		[47:0]	fifo_wr_data_int;
				
wire	fifo_wr_data_mask;		
reg		fifo_wr_data_mask_d1;
assign	fifo_wr_data_mask = (count == 4'hB)? 1'b0 : 1'b1;

//Sync reset to Avalon TX_CLK
altera_std_synchronizer	#(2) reset_tx_clk_sync_u0 (
	.clk			(TX_CLK_IN), 
	.reset_n		(!RESET), 
	.din			(1'b1), 
	.dout           (reset_tx_clk_sync_n)
	);

//Sync reset to Avalon RX_CLK
altera_std_synchronizer	#(2) reset_rx_clk_sync_u0 (
	.clk			(RX_CLK_IN), 
	.reset_n		(!RESET), 
	.din			(1'b1), 
	.dout           (reset_rx_clk_sync_n)
	);

//Sync AVALON_ST_LOOPBACK_ENA to Avalon TX_CLK
altera_std_synchronizer	#(2) avalon_st_lb_ena_sync_u0 (
	.clk			(TX_CLK_IN), 
	.reset_n		(reset_tx_clk_sync_n), 
	.din			(AVALON_ST_LOOPBACK_ENA), 
	.dout           (avalon_st_loopback_ena_sync)
	);
	
//Sync SRC_DST_ADDR_SWAP to Avalon TX_CLK					//jier, address swap not yet implement
altera_std_synchronizer	#(2) src_dst_addr_swap_sync_u0 (
	.clk			(TX_CLK_IN), 
	.reset_n		(reset_tx_clk_sync_n), 
	.din			(SRC_DST_ADDR_SWAP), 
	.dout           (src_dst_addr_swap_sync)
	);
		
always @(posedge TX_CLK_IN or posedge reset_tx_clk_sync)
	if (reset_tx_clk_sync) begin
		TX_SOP_OUT			<= 1'b0;
		TX_DATA_OUT  		<= {32-data_width{1'b0}};
		TX_EOP_OUT			<= 1'b0;
		TX_ERR_OUT			<= 1'b0;
		TX_WREN_OUT			<= 1'b0;
		TX_MOD_OUT			<= 2'b00;
		TX_CRC_FWD_OUT		<= 1'b0;
		RX_RDY_OUT			<= 1'b0;
	end
	else if (!avalon_st_loopback_ena_sync) begin
		TX_SOP_OUT			<= 1'b0;
		TX_DATA_OUT  		<= {32-data_width{1'b0}};
		TX_EOP_OUT			<= 1'b0;
		TX_ERR_OUT			<= 1'b0;
		TX_WREN_OUT			<= 1'b0;
		TX_MOD_OUT			<= 2'b00;
		TX_CRC_FWD_OUT		<= 1'b0;
		RX_RDY_OUT			<= 1'b0;
	end
	else begin
		TX_SOP_OUT			<= 	fifo_rd_data[42];		
		TX_DATA_OUT  		<= 	fifo_rd_data[41:10];	
		TX_EOP_OUT			<= 	fifo_rd_data[9];		
		TX_ERR_OUT			<= 	|fifo_rd_data[8:3];		
		TX_WREN_OUT			<= 	fifo_rd_data[2];		
		TX_MOD_OUT			<= 	fifo_rd_data[1:0];		
		TX_CRC_FWD_OUT		<= 	TX_FWD_CRC;
		RX_RDY_OUT			<=	fifo_rd_data[43];
	end

`ifdef STRATIXIVGX	
fifo_48		fifo_48_u0 (
	.aclr				(reset_rx_clk_sync),
	.wrclk				(RX_CLK_IN),
	.wrreq				(1'b1),
	.data				(fifo_wr_data_int),
	.rdclk				(TX_CLK_IN),
	.rdreq				(1'b1),
	.q					(fifo_rd_data),
	.rdempty			(),
	.wrfull				()
	);
	
`else	//ARRIAIIGX
		
fifo_48_arria		fifo_48_u0 (	
	.aclr				(reset_rx_clk_sync),
	.wrclk				(RX_CLK_IN),
	.wrreq				(1'b1),
	.data				(fifo_wr_data_int),
	.rdclk				(TX_CLK_IN),
	.rdreq				(1'b1),
	.q					(fifo_rd_data),
	.rdempty			(),
	.wrfull				()
	);	
	
`endif //ARRIAIIGX
	
	
// FIFO Write data input delay
always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync) begin
		fifo_wr_data_d1		<=	{48{1'b0}};
		fifo_wr_data_d2		<=	{48{1'b0}};
		fifo_wr_data_d3		<=	{48{1'b0}};
		fifo_wr_data_d4		<=	{48{1'b0}};
		fifo_wr_data_d5		<=	{48{1'b0}};
		fifo_wr_data_d6		<=	{48{1'b0}};
		fifo_wr_data_d7		<=	{48{1'b0}};
		fifo_wr_data_d8		<=	{48{1'b0}};
		fifo_wr_data_d9		<=	{48{1'b0}};
		fifo_wr_data_d10	<=	{48{1'b0}}; 
		fifo_wr_data_d11	<=	{48{1'b0}}; 
		fifo_wr_data_d12	<=	{48{1'b0}};
	end
	else begin
		fifo_wr_data_d1		<=	fifo_wr_data;
		fifo_wr_data_d2		<=	fifo_wr_data_d1;
		fifo_wr_data_d3		<=	fifo_wr_data_d2;
		fifo_wr_data_d4		<=	fifo_wr_data_d3;
		fifo_wr_data_d5		<=	fifo_wr_data_d4;
		fifo_wr_data_d6		<=	fifo_wr_data_d5;
		fifo_wr_data_d7		<=	fifo_wr_data_d6;
		fifo_wr_data_d8		<=	fifo_wr_data_d7;
		fifo_wr_data_d9		<=	fifo_wr_data_d8;
		fifo_wr_data_d10	<=	fifo_wr_data_d9; 
		fifo_wr_data_d11	<=	fifo_wr_data_d10; 
		fifo_wr_data_d12	<=	fifo_wr_data_d11;
	end	
		 	
// Source and Destination address Swap
always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync) begin
		count <= 4'hB;	
	end
	else if (data_width == 8) begin
		if (RX_SOP_IN && RX_DVAL_IN)
			count <= 4'h0;
		else if (RX_DVAL_IN && count == 5'hB)
			count <= count;
		else if (RX_DVAL_IN)
			count <= count + 1'b1;
	end
	else begin	//32bits
		if (RX_SOP_IN && RX_DVAL_IN)
			count <= 4'h3;
		else if (count == 5'hB)
			count <= count;
		else if (RX_DVAL_IN && count == 5'h7)
			count <= 4'hB;
		else if (RX_DVAL_IN)
			count <= count + 4'h4;
	end

always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync) begin
		addr_1 <= {48{1'b0}};
		addr_2 <= {48{1'b0}};
		addr_3 <= {48{1'b0}};
		addr_4 <= {48{1'b0}};
		addr_5 <= {48{1'b0}};
		addr_6 <= {48{1'b0}};
		addr_7 <= {48{1'b0}};
		addr_8 <= {48{1'b0}};
		addr_9 <= {48{1'b0}};
		addr_10 <= {48{1'b0}};
		addr_11 <= {48{1'b0}};
		addr_12 <= {48{1'b0}};
	end
	else if (data_width == 8) begin
		if (RX_SOP_IN && count == 4'hB)
			addr_1	<= fifo_wr_data;
		else if (count == 4'h0)
			addr_2	<= fifo_wr_data;
		else if (count == 4'h1)
			addr_3	<= fifo_wr_data;
		else if (count == 4'h2)
			addr_4	<= fifo_wr_data;
		else if (count == 4'h3)
			addr_5	<= fifo_wr_data;
		else if (count == 4'h4)
			addr_6	<= fifo_wr_data;
		else if (count == 4'h5)
			addr_7	<= fifo_wr_data;
		else if (count == 4'h6)
			addr_8	<= fifo_wr_data;	
		else if (count == 4'h7)
			addr_9	<= fifo_wr_data;
		else if (count == 4'h8)
			addr_10	<= fifo_wr_data;
		else if (count == 4'h9)
			addr_11	<= fifo_wr_data;		
		else if (!RX_SOP_IN && count == 4'hA)
			addr_12	<= fifo_wr_data;
	end
	else begin	//32bits
		if (RX_SOP_IN && count == 4'hB)
			addr_1	<= fifo_wr_data;
		else if (count == 4'h3)
			addr_2	<= fifo_wr_data;
		else if (count == 4'h7)
			addr_3	<= fifo_wr_data;
	end

always @(*)
	if (!fifo_wr_data_mask && fifo_wr_data_mask_d1)
		addr_swap_ena <= 1'b1;
	else if ((addr_swap_count == 4'h0) && (count == 4'hB))
		addr_swap_ena <= 1'b0;
	else
		addr_swap_ena <= addr_swap_ena_d1;
		
always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync)
		fifo_wr_data_mask_d1 <= 1'b0;
	else
		fifo_wr_data_mask_d1 <= fifo_wr_data_mask;			
		
always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync)
		addr_swap_ena_d1 <= 1'b0;
	else
		addr_swap_ena_d1 <= addr_swap_ena;	
	

always @(posedge RX_CLK_IN or posedge reset_rx_clk_sync)
	if (reset_rx_clk_sync)
		addr_swap_count <= 4'h0;
	else if (data_width == 8) begin	
		if (addr_swap_count == 4'hB)
			addr_swap_count <= 4'h0;
		else if (addr_swap_ena)
			addr_swap_count <= addr_swap_count + 1'b1;
	end
	else begin	//32bits
		if (addr_swap_count == 4'h2)
			addr_swap_count <= 4'h0;
		else if (addr_swap_ena)
			addr_swap_count <= addr_swap_count + 1'b1;	
	end
	
always @(*)
	if (data_width == 8) begin
		if (src_dst_addr_swap_sync) begin
				casex ({fifo_wr_data_mask, addr_swap_ena, addr_swap_count})
					6'h10:	
						fifo_wr_data_int = {addr_1[47:42], addr_7[41:10], addr_1[9:0]};
					6'h11:
						fifo_wr_data_int = {addr_2[47:42], addr_8[41:10], addr_2[9:0]};
					6'h12:
						fifo_wr_data_int = {addr_3[47:42], addr_9[41:10], addr_3[9:0]};
					6'h13:
						fifo_wr_data_int = {addr_4[47:42], addr_10[41:10], addr_4[9:0]};
					6'h14:
						fifo_wr_data_int = {addr_5[47:42], addr_11[41:10], addr_5[9:0]};
					6'h15:
						fifo_wr_data_int = {addr_6[47:42], addr_12[41:10], addr_6[9:0]};
					6'h16:
						fifo_wr_data_int = {addr_7[47:42], addr_1[41:10], addr_7[9:0]};
					6'h17:
						fifo_wr_data_int = {addr_8[47:42], addr_2[41:10], addr_8[9:0]};				
					6'h18:
						fifo_wr_data_int = {addr_9[47:42], addr_3[41:10], addr_9[9:0]};	
					6'h19:
						fifo_wr_data_int = {addr_10[47:42], addr_4[41:10], addr_10[9:0]};	
					6'h1A:
						fifo_wr_data_int = {addr_11[47:42], addr_5[41:10], addr_11[9:0]};	
					6'h1B:
						fifo_wr_data_int = {addr_12[47:42], addr_6[41:10], addr_12[9:0]};	
					6'b1?????:
						fifo_wr_data_int = 48'h080000000000;	//    {48{1'b0}};
					default:
						fifo_wr_data_int = fifo_wr_data_d12;
				endcase
		end
		else begin
				casex ({fifo_wr_data_mask, addr_swap_ena, addr_swap_count})
					6'h10:	
						fifo_wr_data_int = addr_1[47:0];
					6'h11:                             
						fifo_wr_data_int = addr_2[47:0];
					6'h12:                              
						fifo_wr_data_int = addr_3[47:0];
					6'h13:                              
						fifo_wr_data_int = addr_4[47:0];
					6'h14:                              
						fifo_wr_data_int = addr_5[47:0];
					6'h15:                              
						fifo_wr_data_int = addr_6[47:0];
					6'h16:                              
						fifo_wr_data_int = addr_7[47:0];
					6'h17:                              
						fifo_wr_data_int = addr_8[47:0];			
					6'h18:                              
						fifo_wr_data_int = addr_9[47:0];
					6'h19:
						fifo_wr_data_int = addr_10[47:0];	
					6'h1A:
						fifo_wr_data_int = addr_11[47:0];	
					6'h1B:
						fifo_wr_data_int = addr_12[47:0];	
					6'b1?????:
						fifo_wr_data_int = 48'h080000000000;	//    {48{1'b0}};
					default:
						fifo_wr_data_int = fifo_wr_data_d12;
				endcase
		end
	end
	else begin	//32bits
		if (src_dst_addr_swap_sync) begin
				casex ({fifo_wr_data_mask, addr_swap_ena, addr_swap_count})
					6'h10:	
						fifo_wr_data_int = {addr_1[47:42], addr_2[25:10], addr_3[41:26], addr_1[9:0]};
					6'h11:
						fifo_wr_data_int = {addr_2[47:42], addr_3[25:10], addr_1[41:26], addr_2[9:0]};
					6'h12:
						fifo_wr_data_int = {addr_3[47:42], addr_1[25:10], addr_2[41:26], addr_3[9:0]};
					6'b1?????:
						fifo_wr_data_int = 48'h080000000000;	//    {48{1'b0}};
					default:
						fifo_wr_data_int = fifo_wr_data_d3;
				endcase
		end
		else begin
				casex ({fifo_wr_data_mask, addr_swap_ena, addr_swap_count})
					6'h10:	
						fifo_wr_data_int = addr_1[47:0];
					6'h11:                             
						fifo_wr_data_int = addr_2[47:0];
					6'h12:                              
						fifo_wr_data_int = addr_3[47:0];
					6'b1?????:
						fifo_wr_data_int = 48'h080000000000;	//    {48{1'b0}};
					default:
						fifo_wr_data_int = fifo_wr_data_d3;
				endcase
		end	
	end
						
endmodule