`timescale 1ns / 1ns
module tb;
   reg    clk;
   reg	reset;
   reg	[3:0] address;
	reg 	[31:0] writedata;
	reg   tx_ready;
	reg write;
	reg read;
	
	wire 	[31:0] readdata;
	
	wire 	[31:0] tx_data;
	wire 	tx_wren;
	wire 	tx_sop;
	wire 	tx_eop;
	wire 	[1:0] tx_mod;
	
	wire	tx_err;
	
	
	reg	[2:0] address1;
	reg 	[31:0] writedata1;
	reg write1;
	reg read1;
	
	wire [31:0] rx_data ;
	wire rx_dval;
	wire rx_sop ;
	wire rx_eop ;
	wire [1:0] rx_mod;
	wire [5:0]rx_err ;
	wire 	[31:0] readdata1;
	wire rx_rdy;
	
	parameter idle = 0;
   parameter set_num_packet = 1;
	parameter config_setting = 2;
	parameter set_random_seed0 = 3;
	parameter set_random_seed1 = 4;
	parameter set_src_addr_0 = 5;
	parameter set_src_addr_1 =6;
	parameter set_dest_addr_0 =7;
	parameter set_dest_addr_1 = 8;
	parameter start_gen = 9;
   parameter set_num_packet_mon = 10;
	parameter start_mon = 11;
	
	reg     [4:0] state; 
	reg     [4:0] nextstate;  
   reg  start;
	
	parameter NUMBER_PACKET		=10000;		//# Number of packets
	parameter ETH_GEN 			=1;				//# Off = 0 & On = 1
	parameter LENGTH_SELECT 	=1;				//# Fixed = 0	& Random = 1	
	parameter PKT_LENGTH 		=64;			//# Ranges from 24 to 9600
	parameter PATTERN_SELECT 	=	1;			//	# Incremental = 0 & Random = 1	
	parameter RANDOM_SEED 		=	46'h 123456789ABC;	//# random seed
	parameter SOURCE_ADDR 		=48'h EE1122334450;	//# MAC destination address
	parameter DEST_ADDR		  =48'h EE1122334450;	//# MAC source address

 
   wire 	pattern_sel;				// Select what type of data pattern: random or incremental
	wire 	length_sel;				// Select what type of packet length: random or fixed
	wire 	[13:0] pkt_length;			// Fixed payload length for every packet
	wire 	[47:0] d_addr;				// Destination MAC address
	wire 	[47:0] s_addr;				// Source MAC address
	wire	[45:0] random_seed;			// Random seed number for PRBS generator
	wire [31:0] num_packet;  
  
  assign num_packet = NUMBER_PACKET;
  assign pattern_sel = PATTERN_SELECT;
  assign length_sel = LENGTH_SELECT;
  assign pkt_length = PKT_LENGTH;
  assign random_seed = RANDOM_SEED;
  assign s_addr = SOURCE_ADDR;
  assign d_addr = DEST_ADDR;
  
  initial
    clk = 1'b1;
  always
    #10 clk <= ~clk;

  initial begin
    tx_ready <= 1'b1;
	 //rx_dval <= 1'b1;
	 reset = 1'b1;
	 #(5*20)
	 reset <=1'b0;
  end

always @(posedge clk or reset)
 begin
   if (reset)
	   begin
		  state <= idle;
		  start <= 1'b1;
		 end
	else
	  state <= nextstate;

 end 
  
  
 always @(nextstate)
 begin 
//  
   if (nextstate == idle)
      begin
        write <= #(2)1'b 0;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h0;  
        writedata <= #(2) 32'h0000_0000;
		  
		  write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
   else if (nextstate == set_num_packet)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h0;
        writedata <= #(2) num_packet;
		 write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == config_setting)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h1;
        writedata <= #(2) {pattern_sel,pkt_length,length_sel};
		   write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
  else if (nextstate == set_random_seed0)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h2;
        writedata <= #(2) random_seed[31:0];
		  
		    write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == set_random_seed1)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h3;
        writedata <= #(2) {19'h0000,random_seed[45:32]};
		   write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == set_src_addr_0)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h4;
        writedata <= #(2) s_addr[31:0];
		  
		   write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == set_src_addr_1)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h5;
        writedata <= #(2) {16'h0000,s_addr[47:32]};
		   write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == set_dest_addr_0)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h6;
        writedata <= #(2) d_addr[31:0];
		  
		    write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == set_dest_addr_1)
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h7;
        writedata <= #(2) {16'h0000,d_addr[47:32]};
		  
		   write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
	else if (nextstate == start_gen )
      begin
        write <= #(2)1'b 1;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h8;  
        writedata <= #(2) 32'h0000_0001;
		  
		    write1 <= #(2)1'b 0;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) 32'h0000_0000;
      end
		
	 else if (nextstate == set_num_packet_mon)
      begin
        
		  write <= #(2)1'b 0;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h0;  
        writedata <= #(2) 32'h0000_0000;
		  
		  write1 <= #(2)1'b 1;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h0;
        writedata1 <= #(2) num_packet;
      end
	else if (nextstate == start_mon)
      begin
          write <= #(2)1'b 0;    
        read <= #(2)1'b 0;    
        address <= #(2) 4'h0;  
        writedata <= #(2) 32'h0000_0000;
		  
		  write1 <= #(2)1'b 1;    
        read1 <= #(2)1'b 0;    
        address1 <= #(2) 3'h7;
        writedata1 <= #(2) 32'h0000_0001;
      end
 end 
//==================================================================	
always @(state or tx_ready or rx_dval)
 begin 
   nextstate <= state;
	case (state)
   idle:
	  begin
      if (tx_ready && start)
     			nextstate <=  set_num_packet;   
      end
	
	set_num_packet:
      begin
      start <= 1'b0;
		if (tx_ready)
     			nextstate <=  config_setting;   
      end
   config_setting:
     
		begin
       start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <=  set_random_seed0;   
         end
      end
    
	 set_random_seed0:
   
	  begin
      start <= 1'b0;
	  if (tx_ready)
         begin
         nextstate <=  set_random_seed1;   
         end
      end
		
		set_random_seed1:
     	begin
     start <= 1'b0; 
	  if (tx_ready)
         begin
         nextstate <= set_src_addr_0;   
         end
      end
		
		set_src_addr_0:
      begin
      start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= set_src_addr_1;   
         end
     
      end
		
		set_src_addr_1:
      begin
      start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= set_dest_addr_0;   
         end
      
      end
	  set_dest_addr_0:
     begin
       start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= set_dest_addr_1;   
         end
      
      end
		
		set_dest_addr_1:
      begin
       start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= start_gen;   
         end
      end
	   
		start_gen:
      begin
		start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= set_num_packet_mon;   
         end
      end
	   
		set_num_packet_mon:
      begin
		start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= start_mon;   
         end
      end
		
		start_mon:
      begin
		start <= 1'b0;
		if (tx_ready)
         begin
         nextstate <= idle;   
         end
      end
	  
	 endcase
 end 

 
 //==============================================================================
 
 eth_gen eth_gen_dut (
   .clk (clk),
	.reset (reset),
	.address (address),
	.write (write),
	.read (read),
	.writedata (writedata),
	.readdata (readdata),
	.tx_rdy (tx_ready),
	.tx_data (tx_data),
	.tx_wren (tx_wren),
	.tx_sop (tx_sop),
	.tx_eop (tx_eop),
	.tx_mod(tx_mod),
	.tx_err (tx_err)
);


assign rx_data = tx_data;
assign rx_dval = tx_wren;
assign rx_sop = tx_sop;
assign rx_eop = tx_eop;
assign rx_mod = tx_mod;
assign rx_err  = 5'b00000;
 
eth_mon eth_mon_dut(
	.clk (clk),
	.reset (reset),
	.address (address1),
	.write (write1),
	.read (read1),
	.writedata (writedata1),
	.readdata (readdata1),
	.rx_data (rx_data),
	.rx_dval (rx_dval),
	.rx_sop (rx_sop),
	.rx_eop (rx_eop),
	.rx_mod (rx_mod),
	.rx_err (rx_err),
	.rx_rdy (rx_rdy)
);

endmodule


