module mii_rate_adaption_fifo (
	TX_CLK,
	RX_CLK,
	TX_CLKENA,
	RX_CLKENA,
	TX_CLK_RESET_N,
	RX_CLK_RESET_N,
	
	//GMII
	GMII_TX_ENA,
	GMII_TX_DATA,
	GMII_TX_ERR,
	
	GMII_RX_ENA,
	GMII_RX_DATA,
	GMII_RX_ERR,
	
	//MII
	MII_TX_ENA,
	MII_TX_DATA,
	MII_TX_ERR,
	
	MII_RX_ENA,
	MII_RX_DATA,
	MII_RX_ERR,
	
	//Status
	GMII_ERROR,
	MII_ERROR
);
//GMII Parameter
parameter	GMII_DATA_WIDTH 		= 8;
parameter	GMII_FIFO_ADDR_WIDTH	= 10;
parameter	GMII_FIFO_DATA_WIDTH	= 10;
//MII Parameter
parameter	MII_DATA_WIDTH			= 4;
parameter	MII_FIFO_ADDR_WIDTH		= 10;
parameter	MII_FIFO_DATA_WIDTH		= 6;
//PPM Detector Parameter
parameter	PPM_COUNT_SIZE = 6; 

parameter	RD_DLY_THRESHOLD = 128;


//Input and Output Ports
input								TX_CLK;
input								RX_CLK;
input								TX_CLKENA;
input								RX_CLKENA;
input								TX_CLK_RESET_N;
input								RX_CLK_RESET_N;
	                    			
//GMII                  			
output								GMII_TX_ENA;
output	[GMII_DATA_WIDTH-1:0]		GMII_TX_DATA;
output								GMII_TX_ERR;
	                    			
input								GMII_RX_ENA;
input	[GMII_DATA_WIDTH-1:0]		GMII_RX_DATA;
input								GMII_RX_ERR;
	                            	
//MII                           	
output								MII_TX_ENA;
output	[MII_DATA_WIDTH-1:0]		MII_TX_DATA;
output								MII_TX_ERR;
	                    			
input								MII_RX_ENA;
input	[MII_DATA_WIDTH-1:0]		MII_RX_DATA;
input								MII_RX_ERR;
	                    			
//Status                			
output								GMII_ERROR;
output								MII_ERROR;
                                	
wire								tx_clk_ena_sync;
wire								rx_clk_ena_sync;

//fifo interface wire
wire								gmii_fifo_wr_ena;
wire	[GMII_FIFO_ADDR_WIDTH-1:0]	gmii_fifo_wr_addr;
wire	[GMII_FIFO_DATA_WIDTH-1:0]	gmii_fifo_wr_data;

wire								gmii_fifo_rd_ena;
wire	[GMII_FIFO_ADDR_WIDTH-1:0]	gmii_fifo_rd_addr;
//wire	[GMII_FIFO_DATA_WIDTH-1:0]	gmii_fifo_rd_data;
wire	[15:0]						gmii_fifo_rd_data;

wire								mii_fifo_wr_ena;
wire	[MII_FIFO_ADDR_WIDTH-1:0]	mii_fifo_wr_addr;
wire	[MII_FIFO_DATA_WIDTH-1:0]	mii_fifo_wr_data;

wire								mii_fifo_rd_ena;
wire	[MII_FIFO_ADDR_WIDTH-1:0]	mii_fifo_rd_addr;
//wire	[MII_FIFO_DATA_WIDTH-1:0]	mii_fifo_rd_data;
wire	[15:0]						mii_fifo_rd_data;

wire								async_reset;
wire								async_reset_n;
        							
wire								gmii_fifo_rdempty;
wire								gmii_fifo_wrfull;
        							
wire								mii_fifo_rdempty;
wire								mii_fifo_wrfull;

assign	async_reset_n = TX_CLK_RESET_N & RX_CLK_RESET_N;
assign	async_reset = !async_reset_n;

assign	GMII_ERROR = gmii_fifo_rdempty | gmii_fifo_wrfull;
assign	MII_ERROR = mii_fifo_rdempty | mii_fifo_wrfull;    	

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Enable Synchronization
//////////////////////////////////////////////////////////////////////////////////////////////////////	
altera_std_synchronizer #(2)  enable_tx_clk_sync (
	.clk			(TX_CLK),
	.reset_n		(TX_CLK_RESET_N),
	.din			(1'b1),
	.dout			(tx_clk_ena_sync)
	);
	
altera_std_synchronizer #(2)  enable_rx_clk_sync (
	.clk			(RX_CLK),
	.reset_n		(RX_CLK_RESET_N),
	.din			(1'b1),
	.dout			(rx_clk_ena_sync)
	);	

//////////////////////////////////////////////////////////////////////////////////////////////////////
//GMII Loopback FIFO
//////////////////////////////////////////////////////////////////////////////////////////////////////

mii_rate_adaption_fifo_ctrl	#(GMII_DATA_WIDTH, GMII_FIFO_ADDR_WIDTH, GMII_FIFO_DATA_WIDTH, PPM_COUNT_SIZE, RD_DLY_THRESHOLD)	gmii_rate_adap_fifo_ctrl_u0 (
	.TX_CLK_ENABLE				(tx_clk_ena_sync),
	.RX_CLK_ENABLE				(rx_clk_ena_sync),
	.TX_CLK_RESET_N				(TX_CLK_RESET_N),
	.RX_CLK_RESET_N				(RX_CLK_RESET_N),

	//MII or GMII interface
	.MII_TX_CLK_IN				(TX_CLK),
	.MII_TX_CLK_ENA				(TX_CLKENA),
	.MII_TX_ENA_OUT				(GMII_TX_ENA),
	.MII_TX_DATA_OUT			(GMII_TX_DATA),
	.MII_TX_ERR_OUT				(GMII_TX_ERR),
	
	.MII_RX_CLK_IN				(RX_CLK),
	.MII_RX_CLK_ENA				(RX_CLKENA),
	.MII_RX_ENA_IN				(GMII_RX_ENA),
	.MII_RX_DATA_IN				(GMII_RX_DATA),
	.MII_RX_ERR_IN				(GMII_RX_ERR),
	
	//RAM or FIFO Interface
	.FIFO_WR_ENA_OUT			(gmii_fifo_wr_ena),
	.FIFO_WR_ADDR_OUT			(gmii_fifo_wr_addr),
	.FIFO_WR_DATA_OUT			(gmii_fifo_wr_data),
	
	.FIFO_RD_ENA_OUT			(gmii_fifo_rd_ena),
	.FIFO_RD_ADDR_OUT			(gmii_fifo_rd_addr),
	.FIFO_RD_DATA_IN			(gmii_fifo_rd_data[GMII_FIFO_DATA_WIDTH-1:0])
);

dp_ram	gmii_dp_ram_u0 (
	//write
	.wrclock					(RX_CLK),	
	.wren						(gmii_fifo_wr_ena),
	.wraddress					(gmii_fifo_wr_addr[7:0]),
	.data						({{6{1'b0}}, gmii_fifo_wr_data}),
	
	//read	
	.rdclock					(TX_CLK),
	.rden						(gmii_fifo_rd_ena),
	.rdaddress					(gmii_fifo_rd_addr[7:0]),
	.q							(gmii_fifo_rd_data),
	
	.rd_aclr					(1'b0)
	);

//////////////////////////////////////////////////////////////////////////////////////////////////////
//MII Loopback FIFO
//////////////////////////////////////////////////////////////////////////////////////////////////////

mii_rate_adaption_fifo_ctrl	#(MII_DATA_WIDTH, MII_FIFO_ADDR_WIDTH, MII_FIFO_DATA_WIDTH, PPM_COUNT_SIZE, RD_DLY_THRESHOLD)	mii_rate_adap_fifo_ctrl_u0 (
	.TX_CLK_ENABLE				(tx_clk_ena_sync),
	.RX_CLK_ENABLE				(rx_clk_ena_sync),
	.TX_CLK_RESET_N				(TX_CLK_RESET_N),
	.RX_CLK_RESET_N				(RX_CLK_RESET_N),

	//MII or GMII interface
	.MII_TX_CLK_IN				(TX_CLK),
	.MII_TX_CLK_ENA				(TX_CLKENA),
	.MII_TX_ENA_OUT				(MII_TX_ENA),
	.MII_TX_DATA_OUT			(MII_TX_DATA),
	.MII_TX_ERR_OUT				(MII_TX_ERR),
	
	.MII_RX_CLK_IN				(RX_CLK),
	.MII_RX_CLK_ENA				(RX_CLKENA),	
	.MII_RX_ENA_IN				(MII_RX_ENA),
	.MII_RX_DATA_IN				(MII_RX_DATA),
	.MII_RX_ERR_IN				(MII_RX_ERR),
	
	//RAM or FIFO Interface
	.FIFO_WR_ENA_OUT			(mii_fifo_wr_ena),
	.FIFO_WR_ADDR_OUT			(mii_fifo_wr_addr),
	.FIFO_WR_DATA_OUT			(mii_fifo_wr_data),
	
	.FIFO_RD_ENA_OUT			(mii_fifo_rd_ena),
	.FIFO_RD_ADDR_OUT			(mii_fifo_rd_addr),
	.FIFO_RD_DATA_IN			(mii_fifo_rd_data[MII_FIFO_DATA_WIDTH-1:0])
);

dp_ram	mii_dp_ram_u0 (
	//write
	.wrclock					(RX_CLK),	
	.wren						(mii_fifo_wr_ena),
	.wraddress					(mii_fifo_wr_addr[7:0]),
	.data						({{10{1'b0}}, mii_fifo_wr_data}),
	
	//read	
	.rdclock					(TX_CLK),
	.rden						(mii_fifo_rd_ena),
	.rdaddress					(mii_fifo_rd_addr[7:0]),
	.q							(mii_fifo_rd_data),
	
	.rd_aclr					(1'b0)
	);

endmodule