module mii_rate_adaption_fifo_ctrl	(
	TX_CLK_ENABLE,
	RX_CLK_ENABLE,
	TX_CLK_RESET_N,
	RX_CLK_RESET_N,
	
	//MII or GMII interface
	MII_TX_CLK_IN,
	MII_TX_CLK_ENA,
	MII_TX_ENA_OUT,
	MII_TX_DATA_OUT,
	MII_TX_ERR_OUT,
	
	MII_RX_CLK_IN,
	MII_RX_CLK_ENA,
	MII_RX_ENA_IN,
	MII_RX_DATA_IN,
	MII_RX_ERR_IN,
	
	//RAM or FIFO Interface
	FIFO_WR_ENA_OUT,
	FIFO_WR_ADDR_OUT,
	FIFO_WR_DATA_OUT,
	
	FIFO_RD_ENA_OUT,
	FIFO_RD_ADDR_OUT,
	FIFO_RD_DATA_IN
);

parameter	MII_DATA_WIDTH = 8;
parameter	FIFO_ADDR_WIDTH = 6;
parameter	FIFO_DATA_WIDTH = 10;
parameter	PPM_COUNT_SIZE = 6;
parameter	RD_DLY_THRESHOLD = 16;

input							TX_CLK_ENABLE;
input							RX_CLK_ENABLE;
input							TX_CLK_RESET_N;
input							RX_CLK_RESET_N;

//GMII/MII TX
input							MII_TX_CLK_IN;
input							MII_TX_CLK_ENA;
output							MII_TX_ENA_OUT;
output	[MII_DATA_WIDTH-1:0]	MII_TX_DATA_OUT;
output							MII_TX_ERR_OUT;

//GMII/MII RX
input							MII_RX_CLK_IN;
input							MII_RX_CLK_ENA;
input							MII_RX_ENA_IN;
input	[MII_DATA_WIDTH-1:0]	MII_RX_DATA_IN;
input							MII_RX_ERR_IN;

//RAM or FIFO Interface
output							FIFO_WR_ENA_OUT;
output	[FIFO_ADDR_WIDTH-1:0]	FIFO_WR_ADDR_OUT;
output	[FIFO_DATA_WIDTH-1:0]	FIFO_WR_DATA_OUT;
	
output							FIFO_RD_ENA_OUT;
output	[FIFO_ADDR_WIDTH-1:0]	FIFO_RD_ADDR_OUT;
input	[FIFO_DATA_WIDTH-1:0]	FIFO_RD_DATA_IN;
	
wire							compensate_wr_pointer_ena;
wire							compensate_rd_pointer_ena;

wire	[FIFO_ADDR_WIDTH-1:0]	compensate_wr_pointer;
wire	[FIFO_ADDR_WIDTH-1:0]	compensate_rd_pointer;


rate_adaption_fifo_wr_ctrl #(MII_DATA_WIDTH, FIFO_ADDR_WIDTH, FIFO_DATA_WIDTH, PPM_COUNT_SIZE) rate_adaption_fifo_wr_ctrl_u0 (
	.ENABLE						(RX_CLK_ENABLE),
		
	.CLK						(MII_RX_CLK_IN),
	.CLK_ENA					(MII_RX_CLK_ENA),
	.RESET_N					(RX_CLK_RESET_N),
	
	//From MII/GMII Interface
	.MII_ENA					(MII_RX_ENA_IN),
	.MII_DATA					(MII_RX_DATA_IN),
	.MII_ERR					(MII_RX_ERR_IN),
	
	//To FIFO	
	.FIFO_WR_ADDR				(FIFO_WR_ADDR_OUT),
	.FIFO_WR_ENA				(FIFO_WR_ENA_OUT),
	.FIFO_WR_DATA				(FIFO_WR_DATA_OUT),
	
	//Control
	.PPM_VALID					(compensate_wr_pointer_ena),
	.PPM_COUNT					(compensate_wr_pointer[PPM_COUNT_SIZE-1:0])
);	
	
rate_adaption_fifo_rd_ctrl #(MII_DATA_WIDTH, FIFO_ADDR_WIDTH, FIFO_DATA_WIDTH, PPM_COUNT_SIZE, RD_DLY_THRESHOLD) rate_adaption_fifo_rd_ctrl_u0 (
	.ENABLE						(TX_CLK_ENABLE),
		
	.CLK						(MII_TX_CLK_IN),
	.CLK_ENA					(MII_TX_CLK_ENA),
	.RESET_N					(TX_CLK_RESET_N),
	
	//To MII/GMII Interface
	.MII_ENA					(MII_TX_ENA_OUT),
	.MII_DATA					(MII_TX_DATA_OUT),
	.MII_ERR					(MII_TX_ERR_OUT),
	
	//From FIFO	
	.FIFO_RD_ADDR				(FIFO_RD_ADDR_OUT),
	.FIFO_RD_ENA				(FIFO_RD_ENA_OUT),
	.FIFO_RD_DATA				(FIFO_RD_DATA_IN),
	
	//Control
	.PPM_VALID					(compensate_rd_pointer_ena),
	.PPM_COUNT					(compensate_rd_pointer[PPM_COUNT_SIZE-1:0])
);

mii_rate_adaption_pointer_ctrl	#(FIFO_ADDR_WIDTH, RD_DLY_THRESHOLD, 8)	mii_rate_adaption_pointer_ctrl_u0 (
	.WR_CLK						(MII_RX_CLK_IN),
	.RD_CLK						(MII_TX_CLK_IN),
	.WR_CLK_ENA					(MII_RX_CLK_ENA),
	.RD_CLK_ENA					(MII_TX_CLK_ENA),
	
	.WR_CLK_RESET_N				(RX_CLK_RESET_N),
	.RD_CLK_RESET_N				(TX_CLK_RESET_N),
	
	.FIFO_WR_ADDR				(FIFO_WR_ADDR_OUT),
	.FIFO_RD_ADDR				(FIFO_RD_ADDR_OUT),
	
	.COMPENSATE_WR_POINTER_ENA	(compensate_wr_pointer_ena),
	.COMPENSATE_WR_POINTER		(compensate_wr_pointer),
	
	.COMPENSATE_RD_POINTER_ENA	(compensate_rd_pointer_ena),
	.COMPENSATE_RD_POINTER		(compensate_rd_pointer)
);


endmodule