module mii_rate_adaption_pointer_ctrl (
	WR_CLK,
	RD_CLK,
	WR_CLK_ENA,
	RD_CLK_ENA,
	
	
	WR_CLK_RESET_N,
	RD_CLK_RESET_N,
	
	FIFO_WR_ADDR,
	FIFO_RD_ADDR,
	
	COMPENSATE_WR_POINTER_ENA,
	COMPENSATE_WR_POINTER,
	
	COMPENSATE_RD_POINTER_ENA,
	COMPENSATE_RD_POINTER
);

parameter	FIFO_ADDR_WIDTH = 6;
parameter	RD_DLY_THRESHOLD = 16;
parameter	COUNT_SIZE = 8;

input							WR_CLK;
input							RD_CLK;
input							WR_CLK_ENA;
input							RD_CLK_ENA;
	
input							WR_CLK_RESET_N;
input							RD_CLK_RESET_N;
	
input	[FIFO_ADDR_WIDTH-1:0]	FIFO_WR_ADDR;
input	[FIFO_ADDR_WIDTH-1:0]	FIFO_RD_ADDR;

output							COMPENSATE_WR_POINTER_ENA;
output	[7:0]					COMPENSATE_WR_POINTER;
	
output							COMPENSATE_RD_POINTER_ENA;
output	[7:0]					COMPENSATE_RD_POINTER;


reg		[COUNT_SIZE-1:0]		period_count;
reg								period_ena;

reg		[FIFO_ADDR_WIDTH-1:0]	fifo_wr_addr_reg;
reg		[FIFO_ADDR_WIDTH-1:0]	fifo_rd_addr_reg;

reg		[FIFO_ADDR_WIDTH-1:0]	fifo_rd_addr_wr_clk_reg;

reg								period_ena_wr_clk_d1;
reg								period_ena_rd_clk_d1;

reg								period_ena_wr_clk_dly; 
reg								period_ena_rd_clk_dly; 

wire							period_ena_wr_clk;
wire							period_ena_rd_clk;

wire							period_ena_wr_clk_sig;
wire							period_ena_rd_clk_sig;

wire	[FIFO_ADDR_WIDTH-1:0]	fifo_wr_addr_rd_clk_sync;
wire	[FIFO_ADDR_WIDTH-1:0]	fifo_rd_addr_wr_clk_sync;


assign period_ena_wr_clk_sig = period_ena_wr_clk & !period_ena_wr_clk_d1;  	
assign period_ena_rd_clk_sig = period_ena_rd_clk & !period_ena_rd_clk_d1;  

always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N)
		period_count <= {COUNT_SIZE{1'b0}};
	else if (WR_CLK_ENA)
		period_count <= period_count + 1'b1;
		
always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N)
		period_ena <= 1'b0;
	else if (WR_CLK_ENA) begin
		if (period_count == {COUNT_SIZE{1'b1}})  		
			period_ena <= 1'b1;
		else
			period_ena <= 1'b0;
	end
		
altera_std_synchronizer  #(2) period_ena_wr_clk_sync (
	.clk					(WR_CLK),
	.reset_n				(WR_CLK_RESET_N),
	.din					(period_ena),
	.dout   				(period_ena_wr_clk)
);		
		
altera_std_synchronizer  #(2) period_ena_rd_clk_sync (
	.clk					(RD_CLK),
	.reset_n				(RD_CLK_RESET_N),
	.din					(period_ena),
	.dout   				(period_ena_rd_clk)
);		
		
always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N)
		period_ena_wr_clk_d1 <= 1'b0;
	else if (WR_CLK_ENA) 
		period_ena_wr_clk_d1 <= period_ena_wr_clk;
		
always @(posedge RD_CLK or negedge RD_CLK_RESET_N)
	if (!RD_CLK_RESET_N)
		period_ena_rd_clk_d1 <= 1'b0;
	else if (RD_CLK_ENA) 
		period_ena_rd_clk_d1 <= period_ena_rd_clk;     

always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N)
		fifo_wr_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
	else if (WR_CLK_ENA && period_ena_wr_clk_sig)
		fifo_wr_addr_reg <= FIFO_WR_ADDR;
		
always @(posedge RD_CLK or negedge RD_CLK_RESET_N)
	if (!RD_CLK_RESET_N)
		fifo_rd_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
	else if (RD_CLK_ENA && period_ena_rd_clk_sig)
		fifo_rd_addr_reg <= FIFO_RD_ADDR;
	
altera_std_synchronizer_bundle  #(FIFO_ADDR_WIDTH, 2) fifo_wr_addr_sync_0 (
	.clk					(RD_CLK),
	.reset_n				(RD_CLK_RESET_N),
	.din					(fifo_wr_addr_reg),
	.dout   				(fifo_wr_addr_rd_clk_sync)
);			  	 

altera_std_synchronizer_bundle  #(FIFO_ADDR_WIDTH, 2) fifo_rd_addr_sync_0 (
	.clk					(WR_CLK),
	.reset_n				(WR_CLK_RESET_N),
	.din					(fifo_rd_addr_reg),
	.dout   				(fifo_rd_addr_wr_clk_sync)
);
	
/////////////////////////////////////////////////////////////////////////////

wire	[FIFO_ADDR_WIDTH-1:0]	write_pointer_diff_sig;
wire	[FIFO_ADDR_WIDTH-1:0]	read_pointer_diff_sig;

reg		[7:0]					write_pointer_diff_reg;
reg		[7:0]					read_pointer_diff_reg;

wire 							period_ena_wr_clk_sig_d3;
wire							period_ena_rd_clk_sig_d3;
		
reg								COMPENSATE_WR_POINTER_ENA;
reg								COMPENSATE_RD_POINTER_ENA;	

assign	write_pointer_diff_sig = fifo_wr_addr_reg - fifo_rd_addr_wr_clk_sync;
assign	read_pointer_diff_sig = fifo_wr_addr_rd_clk_sync - fifo_rd_addr_reg;    

assign	COMPENSATE_WR_POINTER = write_pointer_diff_reg;
assign	COMPENSATE_RD_POINTER = read_pointer_diff_reg; 

always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N) begin
		write_pointer_diff_reg <= {8{1'b0}};
	end
	else if (WR_CLK_ENA) begin
		if (write_pointer_diff_sig[7:0] > RD_DLY_THRESHOLD)  				
			write_pointer_diff_reg <= write_pointer_diff_sig[7:0] - RD_DLY_THRESHOLD;
		else
			write_pointer_diff_reg <= {8{1'b0}};
	end

always @(posedge RD_CLK or negedge RD_CLK_RESET_N)                     
	if (!RD_CLK_RESET_N) begin                                                  
		read_pointer_diff_reg <= {8{1'b0}};
	end            
	else if (RD_CLK_ENA) begin       
		if (read_pointer_diff_sig[7:0] < RD_DLY_THRESHOLD)   			       
			read_pointer_diff_reg <= RD_DLY_THRESHOLD - read_pointer_diff_sig[7:0]; 
		else
			read_pointer_diff_reg <= {8{1'b0}}; 	
	end
		
altera_std_synchronizer  #(3) wr_clk_period_ena_delay (
	.clk					(WR_CLK),
	.reset_n				(WR_CLK_RESET_N),
	.din					(period_ena_wr_clk_sig),
	.dout   				(period_ena_wr_clk_sig_d3)
);		
	
altera_std_synchronizer  #(3) rd_clk_period_ena_delay (
	.clk					(RD_CLK),
	.reset_n				(RD_CLK_RESET_N),
	.din					(period_ena_rd_clk_sig),
	.dout   				(period_ena_rd_clk_sig_d3)
);	

always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N) begin
		period_ena_wr_clk_dly <= 1'b0;
	end
	else if (WR_CLK_ENA) begin
		period_ena_wr_clk_dly <= period_ena_wr_clk_sig_d3;
	end  

always @(posedge RD_CLK or negedge RD_CLK_RESET_N)
	if (!RD_CLK_RESET_N) begin
		period_ena_rd_clk_dly <= 1'b0;
	end
	else if (RD_CLK_ENA) begin
		period_ena_rd_clk_dly <= period_ena_rd_clk_sig_d3;
	end  
	
always @(posedge WR_CLK or negedge WR_CLK_RESET_N)
	if (!WR_CLK_RESET_N) begin
		COMPENSATE_WR_POINTER_ENA <= 1'b0;
	end
	else if (WR_CLK_ENA) begin
		if (write_pointer_diff_reg == {8{1'b0}})
			COMPENSATE_WR_POINTER_ENA <= 1'b0;
		else
			COMPENSATE_WR_POINTER_ENA <= period_ena_wr_clk_dly;
	end
		
always @(posedge RD_CLK or negedge RD_CLK_RESET_N)
	if (!RD_CLK_RESET_N) begin
		COMPENSATE_RD_POINTER_ENA <= 1'b0;
	end
	else if (RD_CLK_ENA) begin
		if (read_pointer_diff_reg == {8{1'b0}})
			COMPENSATE_RD_POINTER_ENA <= 1'b0;
		else
			COMPENSATE_RD_POINTER_ENA <= period_ena_rd_clk_dly;
	end
	
endmodule