`define STRATIXIVGX
//`define	ARRIAIIGX

	
module pcs_loopback_mux (
	
	RESET,
	SET_10,
	SET_1000,
	PCS_LOOPBACK_ENA,
	MAC_IF_DIS,
	
	//From MAC MII/GMII Interface
	MAC_TX_CLK,
	MAC_RX_CLK,
	
	MAC_GMII_TX_D,
	MAC_GMII_TX_EN,
	MAC_GMII_TX_ERR,
	
	MAC_GMII_RX_D,
	MAC_GMII_RX_DV,
	MAC_GMII_RX_ERR,
	
	MAC_MII_TX_D,
	MAC_MII_TX_EN,
	MAC_MII_TX_ERR,
	
	MAC_MII_RX_D,
	MAC_MII_RX_DV,
	MAC_MII_RX_ERR,
	MAC_MII_COL,
	MAC_MII_CRS,


	//From 1000Base-X/SGMII PCS and PMA MII/GMII Interface
	PCS_GMII_TX_D,
	PCS_GMII_TX_EN,
	PCS_GMII_TX_ERR,
	
	PCS_GMII_RX_D,
	PCS_GMII_RX_DV,
	PCS_GMII_RX_ERR,
	
	PCS_MII_TX_D,
	PCS_MII_TX_EN,
	PCS_MII_TX_ERR,
	
	PCS_MII_RX_D,
	PCS_MII_RX_DV,
	PCS_MII_RX_ERR,
	PCS_MII_COL,
	PCS_MII_CRS,
	
	PCS_TX_CLKENA,
	PCS_RX_CLKENA,
	
	PCS_TX_CLK,
	PCS_RX_CLK,
	
	PCS_RESET_RX_CLK,
	PCS_RESET_TX_CLK
	);
	
input			RESET;
input			SET_10;
input			SET_1000;
input			PCS_LOOPBACK_ENA;
input			MAC_IF_DIS;
	
//From MAC MII/GMII Interface
output			MAC_TX_CLK;
output			MAC_RX_CLK;

input	[7:0]	MAC_GMII_TX_D;
input			MAC_GMII_TX_EN;
input			MAC_GMII_TX_ERR;
	
output	[7:0]	MAC_GMII_RX_D;
output			MAC_GMII_RX_DV;
output			MAC_GMII_RX_ERR;
	
input	[3:0]	MAC_MII_TX_D;
input			MAC_MII_TX_EN;
input			MAC_MII_TX_ERR;
	
output	[3:0]	MAC_MII_RX_D;
output			MAC_MII_RX_DV;
output			MAC_MII_RX_ERR;
output			MAC_MII_COL;
output			MAC_MII_CRS;

//From 1000Base-X/SGMII PCS and PMA MII/GMII Interface
output	[7:0]	PCS_GMII_TX_D;
output			PCS_GMII_TX_EN;
output			PCS_GMII_TX_ERR;
	
input	[7:0]	PCS_GMII_RX_D;
input			PCS_GMII_RX_DV;
input			PCS_GMII_RX_ERR;
	
output	[3:0]	PCS_MII_TX_D;
output			PCS_MII_TX_EN;
output			PCS_MII_TX_ERR;
	
input	[3:0]	PCS_MII_RX_D;
input			PCS_MII_RX_DV;
input			PCS_MII_RX_ERR;
input			PCS_MII_COL;
input			PCS_MII_CRS;
	
input			PCS_TX_CLKENA;
input			PCS_RX_CLKENA;
	
input			PCS_RX_CLK;
input			PCS_TX_CLK;
		
output			PCS_RESET_RX_CLK;
output			PCS_RESET_TX_CLK;

//////////////////////////////////////////////////////////////////////////////////////////
//Register
reg		[7:0]	PCS_GMII_TX_D;
reg				PCS_GMII_TX_EN;
reg				PCS_GMII_TX_ERR;

reg		[7:0]	MAC_GMII_RX_D;
reg				MAC_GMII_RX_DV;
reg				MAC_GMII_RX_ERR;

reg		[3:0]	PCS_MII_TX_D;
reg				PCS_MII_TX_EN;
reg				PCS_MII_TX_ERR;

reg		[3:0]	MAC_MII_RX_D;
reg				MAC_MII_RX_DV;
reg				MAC_MII_RX_ERR;
reg				MAC_MII_COL;
reg				MAC_MII_CRS;

//////////////////////////////////////////////////////////////////////////////////////////
wire			pcs_loopback_ena_sync;
wire			pcs_mac_if_dis_sync;
wire			reset_rx_clk_sync_n;
wire			reset_tx_clk_sync_n;

//GMII
wire	[9:0]	fifo_10_wr_data;
wire	[9:0]	fifo_10_rd_data;
wire			gmii_fifo_rdempty_status;
wire			gmii_fifo_wrfull_status;

assign	fifo_10_wr_data = {PCS_GMII_RX_DV, PCS_GMII_RX_ERR, PCS_GMII_RX_D};  

//MII
wire	[5:0]	fifo_6_wr_data;
wire	[5:0]	fifo_6_rd_data;
wire			mii_fifo_rdempty_status;
wire			mii_fifo_wrfull_status;

wire			gmii_tx_ena;
wire	[7:0]	gmii_tx_data;
wire			gmii_tx_err;

wire			mii_tx_ena;
wire	[3:0]	mii_tx_data;
wire			mii_tx_err;

wire			pcs_div_txclk;
wire			pcs_div_rxclk;

assign	fifo_6_wr_data = {PCS_MII_RX_DV, PCS_MII_RX_ERR, PCS_MII_RX_D};  

//RESET
assign	PCS_RESET_TX_CLK = !reset_tx_clk_sync_n;
assign	PCS_RESET_RX_CLK = !reset_rx_clk_sync_n;

/////////////////////////////////////////////////////////////////////////////////////////////////////
//Clock Mux (Cautions) 

assign	mac_tx_clk_int = SET_1000? PCS_TX_CLK : pcs_div_txclk;
assign 	mac_rx_clk_int = SET_1000? PCS_RX_CLK : pcs_div_rxclk;

//for Global clock
altclk_buf 		mac_tx_clk_out_u0 (
	.inclk			(mac_tx_clk_int),
	.outclk			(MAC_TX_CLK)
	);
	
altclk_buf 		mac_rx_clk_out_u0 (
	.inclk			(mac_rx_clk_int),
	.outclk			(MAC_RX_CLK)
	);		
	
/////////////////////////////////////////////////////////////////////////////////////////////////////
//Sync reset to PCS_TX_CLK
altera_std_synchronizer	#(2) reset_tx_clk_sync_u0 (
	.clk			(PCS_TX_CLK), 
	.reset_n		(!RESET), 
	.din			(1'b1), 
	.dout           (reset_tx_clk_sync_n)
	);

//Sync reset to PCS_RX_CLK	
altera_std_synchronizer	#(2) reset_rx_clk_sync_u0 (
	.clk			(PCS_RX_CLK), 
	.reset_n		(!RESET), 
	.din			(1'b1), 
	.dout           (reset_rx_clk_sync_n)
	);

//Sync PCS_LOOPBACK_ENA to PCS_TX_CLK 
altera_std_synchronizer	#(2) pcs_loopback_ena_sync_u0 (
	.clk			(PCS_TX_CLK), 
	.reset_n		(reset_tx_clk_sync_n), 
	.din			(PCS_LOOPBACK_ENA), 
	.dout           (pcs_loopback_ena_sync)
	);

//Sync MAC_IF_DIS to PCS_RX_CLK 
altera_std_synchronizer	#(2) pcs_mac_if_dis_sync_u0 (
	.clk			(PCS_RX_CLK), 
	.reset_n		(reset_rx_clk_sync_n), 
	.din			(MAC_IF_DIS), 
	.dout           (pcs_mac_if_dis_sync)
	);
	
/////////////////////////////////////////////////////////////////////////////////////////////////////
// MII Clock Divider (125Mhz, 25Mhz, 2.5Mhz)
/////////////////////////////////////////////////////////////////////////////////////////////////////
//TX Clock Divider
clock_divider	clock_div_tx_u0 (
	.CLK			(PCS_TX_CLK),
	.CLK_ENA		(PCS_TX_CLKENA),
	.RESET_N		(reset_tx_clk_sync_n),

	.SET_10			(SET_10),
		
	.CLK_DIV_OUT	(pcs_div_txclk)
);

//RX Clock Divider
clock_divider	clock_div_rx_u0 (
	.CLK			(PCS_RX_CLK),
	.CLK_ENA		(PCS_RX_CLKENA),
	.RESET_N		(reset_rx_clk_sync_n),

	.SET_10			(SET_10),
		
	.CLK_DIV_OUT	(pcs_div_rxclk)
);

/////////////////////////////////////////////////////////////////////////////////////////////////////
// GMII Interface
/////////////////////////////////////////////////////////////////////////////////////////////////////
// TX

//PCS GMII TX MUX
always @ (posedge PCS_TX_CLK or posedge PCS_RESET_TX_CLK)
	if (PCS_RESET_TX_CLK) begin
		PCS_GMII_TX_EN		<= 1'b0;
		PCS_GMII_TX_ERR		<= 1'b0;
		PCS_GMII_TX_D		<= 8'h00;
	end
	else if (PCS_TX_CLKENA) begin
		if (pcs_loopback_ena_sync) begin
			PCS_GMII_TX_EN		<= gmii_tx_ena;			//pcs_gmii_rx_dv_int;
			PCS_GMII_TX_ERR 	<= gmii_tx_err;			//pcs_gmii_rx_d_int;
			PCS_GMII_TX_D		<= gmii_tx_data;		//pcs_gmii_rx_err_int;
		end
		else begin
			PCS_GMII_TX_EN		<= MAC_GMII_TX_EN;
			PCS_GMII_TX_ERR		<= MAC_GMII_TX_ERR;
			PCS_GMII_TX_D		<= MAC_GMII_TX_D;
		end
	end

mii_rate_adaption_fifo		mii_rate_adaption_fifo_u0 (
	.TX_CLK					(PCS_TX_CLK),
	.RX_CLK					(PCS_RX_CLK),
	.TX_CLKENA				(PCS_TX_CLKENA),
	.RX_CLKENA				(PCS_RX_CLKENA),
	
	.TX_CLK_RESET_N			(reset_tx_clk_sync_n),
	.RX_CLK_RESET_N			(reset_rx_clk_sync_n),
	
	//GMII
	.GMII_TX_ENA			(gmii_tx_ena),
	.GMII_TX_DATA			(gmii_tx_data),
	.GMII_TX_ERR			(gmii_tx_err),
	
	.GMII_RX_ENA			(PCS_GMII_RX_DV),
	.GMII_RX_DATA			(PCS_GMII_RX_D),
	.GMII_RX_ERR			(PCS_GMII_RX_ERR),
	
	//MII
	.MII_TX_ENA				(mii_tx_ena),
	.MII_TX_DATA			(mii_tx_data),
	.MII_TX_ERR				(mii_tx_err),
	
	.MII_RX_ENA				(PCS_MII_RX_DV),
	.MII_RX_DATA			(PCS_MII_RX_D),
	.MII_RX_ERR				(PCS_MII_RX_ERR),
	
	//Status
	.GMII_ERROR				(gmii_error),
	.MII_ERROR				(mii_error)
);
		
/////////////////////////////////////////////////////////////////////////////////////////////////////
// RX
// PCS RX MUX (MAC IF DISABLE)
always @(posedge PCS_RX_CLK or posedge PCS_RESET_RX_CLK)
	if (PCS_RESET_RX_CLK) begin
		MAC_GMII_RX_DV		<= 1'b0;
		MAC_GMII_RX_D		<= 8'h00;
		MAC_GMII_RX_ERR		<= 1'b0;
	end
	else if (pcs_mac_if_dis_sync) begin
		MAC_GMII_RX_DV		<= 1'b0;
		MAC_GMII_RX_D		<= 8'h00;
		MAC_GMII_RX_ERR		<= 1'b0;
	end
	else begin
		MAC_GMII_RX_DV		<= PCS_GMII_RX_DV;
		MAC_GMII_RX_D		<= PCS_GMII_RX_D;
		MAC_GMII_RX_ERR		<= PCS_GMII_RX_ERR;
	end

/////////////////////////////////////////////////////////////////////////////////////////////////////
// MII Interface
/////////////////////////////////////////////////////////////////////////////////////////////////////
// TX

wire	[5:0]	mii_tx_sync;
wire	[5:0]	mii_tx_sync_d2;
assign	mii_tx_sync = {MAC_MII_TX_EN, MAC_MII_TX_ERR, MAC_MII_TX_D};

altera_std_synchronizer_bundle  #(6, 3) mii_tx_if_sync_u0 (
	.clk				(PCS_TX_CLK),
	.reset_n			(reset_tx_clk_sync_n),
	.din				(mii_tx_sync),
	.dout				(mii_tx_sync_d2)
	);
	
//PCS MII TX MUX
always @ (posedge PCS_TX_CLK or posedge PCS_RESET_TX_CLK)
	if (PCS_RESET_TX_CLK) begin
		PCS_MII_TX_EN		<= 1'b0;
		PCS_MII_TX_ERR		<= 1'b0;
		PCS_MII_TX_D		<= 4'h0;
	end
	else if (PCS_TX_CLKENA) begin
		if (pcs_loopback_ena_sync) begin
			PCS_MII_TX_EN		<= mii_tx_ena;				//pcs_mii_rx_dv_int;
			PCS_MII_TX_ERR 		<= mii_tx_err;				//pcs_mii_rx_err_int;
			PCS_MII_TX_D		<= mii_tx_data;				//pcs_mii_rx_d_int;
		end
		else if (PCS_TX_CLKENA) begin
			PCS_MII_TX_EN		<= mii_tx_sync_d2[5];		//MAC_MII_TX_EN;
			PCS_MII_TX_ERR 		<= mii_tx_sync_d2[4];		//MAC_MII_TX_ERR;
			PCS_MII_TX_D		<= mii_tx_sync_d2[3:0];		//MAC_MII_TX_D;
		end
	end

/////////////////////////////////////////////////////////////////////////////////////////////////////	
// RX
wire			reset_mii_rx_clk_sync_n;
wire	[7:0]	mii_rx_sync;
wire	[7:0]	mii_rx_sync_d2;
wire			pcs_mac_if_dis_mii_rx_clk_sync;

assign	mii_rx_sync = {PCS_MII_RX_DV, PCS_MII_RX_ERR, PCS_MII_COL, PCS_MII_CRS, PCS_MII_RX_D};

altera_std_synchronizer	#(3) reset_mii_rx_clk_sync_u0 (
	.clk			(MAC_RX_CLK), 
	.reset_n		(!RESET), 
	.din			(1'b1), 
	.dout           (reset_mii_rx_clk_sync_n)
	);

altera_std_synchronizer_bundle  #(8, 3) mii_rx_if_sync_u0 (
	.clk			(MAC_RX_CLK),
	.reset_n		(reset_mii_rx_clk_sync_n),
	.din			(mii_rx_sync),
	.dout			(mii_rx_sync_d2)
	);
	
//Sync MAC_IF_DIS to mii_rx_clk 
altera_std_synchronizer	#(3) pcs_mac_if_dis_sync_u1 (
	.clk			(MAC_RX_CLK), 
	.reset_n		(reset_mii_rx_clk_sync_n), 
	.din			(MAC_IF_DIS), 
	.dout           (pcs_mac_if_dis_mii_rx_clk_sync)
	);	
	
// PCS RX MUX (MAC IF DISABLE)
always @(posedge MAC_RX_CLK or negedge reset_mii_rx_clk_sync_n)
	if (!reset_mii_rx_clk_sync_n) begin
		MAC_MII_RX_DV		<= 1'b0;
		MAC_MII_RX_ERR		<= 1'b0;
		MAC_MII_COL			<= 1'b0;
		MAC_MII_CRS			<= 1'b0;
		MAC_MII_RX_D		<= 4'h0;
	end
	else if (pcs_mac_if_dis_mii_rx_clk_sync) begin
		MAC_MII_RX_DV		<= 1'b0;
		MAC_MII_RX_ERR		<= 1'b0;
		MAC_MII_COL			<= 1'b0;
		MAC_MII_CRS			<= 1'b0;
		MAC_MII_RX_D		<= 4'h0;
	end
	else begin
		MAC_MII_RX_DV		<= mii_rx_sync_d2[7];			//PCS_MII_RX_DV;
		MAC_MII_RX_ERR		<= mii_rx_sync_d2[6];			//PCS_MII_RX_ERR;
		MAC_MII_COL			<= mii_rx_sync_d2[5];			//PCS_MII_COL;
		MAC_MII_CRS			<= mii_rx_sync_d2[4];			//PCS_MII_CRS;
		MAC_MII_RX_D		<= mii_rx_sync_d2[3:0];			//PCS_MII_RX_D;
	end


/////////////////////////////////////////////////////////////////////////////////////////////////////
	
endmodule