module pcs_loopback_reg (
	CLK,
	RESET,
	ADDRESS,
	WRITE,
	READ,
	WRITEDATA,
	READDATA,
	WAITREQUEST,
	
	PCS_LOOPBACK_ENA,
	MAC_IF_DIS
	);
	
input			CLK;
input			RESET;
input	[3:0]	ADDRESS;
input			WRITE;
input			READ;
input	[15:0]	WRITEDATA;
output	[15:0]	READDATA;
output			WAITREQUEST;
	
output			PCS_LOOPBACK_ENA;
output			MAC_IF_DIS;
	
reg				PCS_LOOPBACK_ENA;
reg				MAC_IF_DIS;
reg		[15:0]	READDATA;
			
assign	WAITREQUEST = 1'b0;

// Register
// ------------------------------------------

always @ (posedge CLK or posedge RESET)
	if (RESET)
		PCS_LOOPBACK_ENA <= 1'b0;
	else if (WRITE && ADDRESS == 4'h0)
		PCS_LOOPBACK_ENA <= WRITEDATA[0];

always @ (posedge CLK or posedge RESET)
	if (RESET)
		MAC_IF_DIS <= 1'b0; 						
	else if (WRITE && ADDRESS == 4'h1)
		MAC_IF_DIS <= WRITEDATA[0];

// Output MUX of registers into readdata bus
// ------------------------------------------

always @ (READ or ADDRESS or PCS_LOOPBACK_ENA or MAC_IF_DIS)
	begin	
		case ({READ,ADDRESS})
			5'h10: 
				READDATA = {{15{1'b0}}, PCS_LOOPBACK_ENA};
			5'h11: 
				READDATA = {{15{1'b0}}, MAC_IF_DIS};
			default: 
				READDATA = 16'h0000;
		endcase
	end		
		
endmodule