module pcs_loopback_wrapper (
	CLK,
	RESET,
	ADDRESS,
	WRITE,
	READ,
	WRITEDATA,
	READDATA,
	WAITREQUEST,
	
	//From MAC MII/GMII Interface
	MAC_TX_CLK,
	MAC_RX_CLK,
	
	MAC_GMII_TX_D,
	MAC_GMII_TX_EN,
	MAC_GMII_TX_ERR,
	
	MAC_GMII_RX_D,
	MAC_GMII_RX_DV,
	MAC_GMII_RX_ERR,
	
	MAC_MII_TX_D,
	MAC_MII_TX_EN,
	MAC_MII_TX_ERR,
	
	MAC_MII_RX_D,
	MAC_MII_RX_DV,
	MAC_MII_RX_ERR,
	MAC_MII_COL,
	MAC_MII_CRS,


	//From 1000Base-X/SGMII PCS and PMA MII/GMII Interface
	PCS_GMII_TX_D,
	PCS_GMII_TX_EN,
	PCS_GMII_TX_ERR,
	
	PCS_GMII_RX_D,
	PCS_GMII_RX_DV,
	PCS_GMII_RX_ERR,
	
	PCS_MII_TX_D,
	PCS_MII_TX_EN,
	PCS_MII_TX_ERR,
	
	PCS_MII_RX_D,
	PCS_MII_RX_DV,
	PCS_MII_RX_ERR,
	PCS_MII_COL,
	PCS_MII_CRS,
	
	PCS_TX_CLKENA,
	PCS_RX_CLKENA,
	
	PCS_RX_CLK,
	PCS_TX_CLK,
	
	PCS_RESET_RX_CLK,
	PCS_RESET_TX_CLK,
	
	SET_10,
	SET_1000
);

input			CLK;
input			RESET;
input	[3:0]	ADDRESS;
input			WRITE;
input			READ;
input	[15:0]	WRITEDATA;
output	[15:0]	READDATA;
output			WAITREQUEST;

//From MAC MII/GMII Interface
output			MAC_TX_CLK;
output			MAC_RX_CLK;

input	[7:0]	MAC_GMII_TX_D;
input			MAC_GMII_TX_EN;
input			MAC_GMII_TX_ERR;
	
output	[7:0]	MAC_GMII_RX_D;
output			MAC_GMII_RX_DV;
output			MAC_GMII_RX_ERR;
	
input	[3:0]	MAC_MII_TX_D;
input			MAC_MII_TX_EN;
input			MAC_MII_TX_ERR;
	
output	[3:0]	MAC_MII_RX_D;
output			MAC_MII_RX_DV;
output			MAC_MII_RX_ERR;
output			MAC_MII_COL;
output			MAC_MII_CRS;

//From 1000Base-X/SGMII PCS and PMA MII/GMII Interface
output	[7:0]	PCS_GMII_TX_D;
output			PCS_GMII_TX_EN;
output			PCS_GMII_TX_ERR;
	
input	[7:0]	PCS_GMII_RX_D;
input			PCS_GMII_RX_DV;
input			PCS_GMII_RX_ERR;
	
output	[3:0]	PCS_MII_TX_D;
output			PCS_MII_TX_EN;
output			PCS_MII_TX_ERR;
	
input	[3:0]	PCS_MII_RX_D;
input			PCS_MII_RX_DV;
input			PCS_MII_RX_ERR;
input			PCS_MII_COL;
input			PCS_MII_CRS;
	
input			PCS_TX_CLKENA;
input			PCS_RX_CLKENA;
	
input			PCS_RX_CLK;
input			PCS_TX_CLK;
		
output			PCS_RESET_RX_CLK;
output			PCS_RESET_TX_CLK;

input			SET_10;
input			SET_1000;

wire			loopback_ena;
wire			mac_if_dis;

pcs_loopback_reg	pcs_loopback_reg_u0 (
	.CLK					(CLK),
	.RESET					(RESET),
	.ADDRESS				(ADDRESS),
	.WRITE					(WRITE),
	.READ					(READ),
	.WRITEDATA				(WRITEDATA),
	.READDATA				(READDATA),
	.WAITREQUEST			(WAITREQUEST),
	
	.PCS_LOOPBACK_ENA		(loopback_ena),
	.MAC_IF_DIS				(mac_if_dis)
	);


pcs_loopback_mux	pcs_loopback_mux_u0 (
	
	.RESET					(RESET),
	.SET_10					(SET_10),
	.SET_1000				(SET_1000),
	.PCS_LOOPBACK_ENA		(loopback_ena),
	.MAC_IF_DIS				(mac_if_dis),
	
	//From MAC MII/GMII Interface
	.MAC_TX_CLK				(MAC_TX_CLK),
	.MAC_RX_CLK				(MAC_RX_CLK),
	
	.MAC_GMII_TX_D			(MAC_GMII_TX_D),
	.MAC_GMII_TX_EN			(MAC_GMII_TX_EN),
	.MAC_GMII_TX_ERR		(MAC_GMII_TX_ERR),
	
	.MAC_GMII_RX_D			(MAC_GMII_RX_D),
	.MAC_GMII_RX_DV			(MAC_GMII_RX_DV),
	.MAC_GMII_RX_ERR		(MAC_GMII_RX_ERR),
	
	.MAC_MII_TX_D			(MAC_MII_TX_D),
	.MAC_MII_TX_EN			(MAC_MII_TX_EN),
	.MAC_MII_TX_ERR			(MAC_MII_TX_ERR),
	
	.MAC_MII_RX_D			(MAC_MII_RX_D),
	.MAC_MII_RX_DV			(MAC_MII_RX_DV),
	.MAC_MII_RX_ERR			(MAC_MII_RX_ERR),
	.MAC_MII_COL			(MAC_MII_COL),
	.MAC_MII_CRS			(MAC_MII_CRS),


	//From 1000Base-X/SGMII PCS and PMA MII/GMII Interface
	.PCS_GMII_TX_D			(PCS_GMII_TX_D),
	.PCS_GMII_TX_EN			(PCS_GMII_TX_EN),
	.PCS_GMII_TX_ERR		(PCS_GMII_TX_ERR),
	
	.PCS_GMII_RX_D			(PCS_GMII_RX_D),
	.PCS_GMII_RX_DV			(PCS_GMII_RX_DV),
	.PCS_GMII_RX_ERR		(PCS_GMII_RX_ERR),
	
	.PCS_MII_TX_D			(PCS_MII_TX_D),
	.PCS_MII_TX_EN			(PCS_MII_TX_EN),
	.PCS_MII_TX_ERR			(PCS_MII_TX_ERR),
	
	.PCS_MII_RX_D			(PCS_MII_RX_D),
	.PCS_MII_RX_DV			(PCS_MII_RX_DV),
	.PCS_MII_RX_ERR			(PCS_MII_RX_ERR),
	.PCS_MII_COL			(PCS_MII_COL),
	.PCS_MII_CRS			(PCS_MII_CRS),
	
	.PCS_TX_CLKENA			(PCS_TX_CLKENA),
	.PCS_RX_CLKENA			(PCS_RX_CLKENA),
	
	.PCS_RX_CLK				(PCS_RX_CLK),
	.PCS_TX_CLK				(PCS_TX_CLK),
	
	.PCS_RESET_RX_CLK		(PCS_RESET_RX_CLK),
	.PCS_RESET_TX_CLK       (PCS_RESET_TX_CLK)
	);


endmodule